# calculate global maps of CN ratios for different runs 
# SR 7.6.2016

library(devtools)
library(magclass)
library(luplot)
library(ncdf)
library(RColorBrewer)
DiePalette <- colorRampPalette(brewer.pal(10, "RdYlBu")[10:1])

#load_all("/p/projects/landuse/useful_scripts/landmark/benchmarking_system/LandMark/R/", 
# reset=TRUE,export_all=TRUE)
#load_all("/p/projects/landuse/useful_scripts/landmark/benchmarking_system/PIKTools/R/", 
# reset=TRUE,export_all=TRUE)

source("read_LPJmL_nc.R")
source("cn_functions.R")
oneplot <- TRUE
usebin  <- FALSE
version <- c("rcp85","rcp85r")[2]
# nl: N limited, nu: N unlimited, cd: climate dynamic, cc: climate constant
paths <- switch(version,
  "rcp85"  = c(nlcd="/p/projects/biodiversity/bloh/nitrogen/rcp85/output/",
               nucd="/p/projects/biodiversity/bloh/nitrogen/rcp85_unlimit/output/",
               nlcc="/p/projects/biodiversity/bloh/nitrogen/rcp85_cc/output/",
               nucc="/p/projects/biodiversity/bloh/nitrogen/rcp85_unlimit_cc/output/",
               trun="/p/projects/biodiversity/bloh/lpj_ref_rcp85/output/"),
  "rcp85r"  = c(nlcd="/p/projects/biodiversity/bloh/nitrogen/rcp85/output/",
               nucd="/p/projects/biodiversity/bloh/nitrogen/rcp85_unlimit/output/",
               nlcc="/p/projects/biodiversity/bloh/nitrogen/resp/rcp85_cc/output/",
               nucc="/p/projects/biodiversity/bloh/nitrogen/rcp85_unlimit_cc/output/",
               trun="/p/projects/biodiversity/bloh/lpj_ref_rcp85/output/"),
  "rcp85o" = c(nlcd="/p/projects/biodiversity/bloh/nitrogen/rcp85/output/",
               nucd="/p/projects/biodiversity/bloh/nitrogen/rcp85_unlimit/output/",
               nlcc="/p/projects/biodiversity/bloh/nitrogen/rcp85_cc/output/",
               nucc="/p/projects/biodiversity/bloh/nitrogen/rcp85_unlimit_cc/output/")
)

calratio <- function(lis,v1,v2,t1,t2,thresh=20,nafin=T){
  k1 <- lis[[v1]][,,t1]
  ii <- which(k1 < thresh)
  if (length(ii) > 0) k1[ii] <- NA
  k2 <- lis[[v2]][,,t2]
  kk <- k2/k1
  if (nafin) {
    ii <- which(!is.finite(kk))
    if (length(ii) > 0) kk[ii] <- NA
  }
  kk
}

vars <- c("mnpp","vegc","soilc")
bands <- c(12,1,1)

print(paths)
print(version)

y1 <- 1901
y2 <- 2099
yy <- c(1905,2000,2050,2090)
ya <- 5
ncell <- 67420

for (k in 1:length(vars)){
  ys <- rep(y1:y2,each=bands[k])
  pfiles <- paste0(vars[k],".nc")
  print(pfiles)
  res_ts <- NULL
  for (m in 1:length(paths)) {
    fil <- paste0(paths[[m]],pfiles[1])
    if (file.exists(fil)){
      print(fil)
      nc <- open.ncdf(paste0(paths[[m]],pfiles[1]))
      vari <- names(nc$var)
      lon <- get.var.ncdf(nc,"longitude")
      lat <-  get.var.ncdf(nc,"latitude")
      ff <- array(NA,dim=c(length(lon),length(lat),length(yy)))
      for (n in 1:length(yy)){
        print(yy[n])
        ii <- which(ys >= yy[n]-ya & ys <= yy[n]+ya)
        dd <- get.var.ncdf(nc,vari,start=c(1,1,ii[1]),count=c(720,280,length(ii)))
        ff[,,n] <- apply(dd,1:2,mean)*bands[k]
      }
      res_ts <- c(res_ts,list(ff))
      names(res_ts)[m] <- names(paths)[m]
      close.ncdf(nc)
    }
  }
  save(res_ts,file=paste0("results/res_map_",vars[k],"_",version,".RData"))

  colo <- rev(DiePalette(51))
  colo[26] <- grey(0.7)
  ## plot now
  pdf(paste0("plot/map_",vars[k],"_2090_2000_",version,".pdf"),height=5,width=8)
  for (v in names(res_ts)){
    aa <-  calratio(res_ts,v,v,2,4)
    kk <- list(x=lon,y=lat,z=aa)
    par(las=1,lwd=2)
    plotmap(kk,legend_range=c(0,2),show_water=F,legend_colours=colo,main=v)
  }
  dev.off()
  
  # plot limitation status
  # limited/unlimited
  coln <- rev(DiePalette(51))
  for (n in 1:length(yy)){
    pdf(paste0("plot/map_ratio_",yy[n],"_",vars[k],"_",version,".pdf"),height=5,width=8)
    par(las=1,lwd=2)
    aa <-  calratio(res_ts,"nucd","nlcd",n,n)
    kk <- list(x=lon,y=lat,z=aa)
    plotmap(kk,legend_range=c(0.5,1),show_water=F,
      main=paste("Dynamic climate",yy[n],vars[k]),legend_colours=coln)
    bb <-  calratio(res_ts,"nucc","nlcc",n,n)
    kk <- list(x=lon,y=lat,z=bb)
    plotmap(kk,legend_range=c(0.5,1),show_water=F,
      main=paste("Constant climate",yy[n],vars[k]),legend_colours=coln)
    cc <- aa/bb
    ii <- which(!is.finite(cc))
    if (length(ii) > 0) cc[ii] <- NA
    kk <- list(x=lon,y=lat,z=cc)
    plotmap(kk,legend_range=c(0,2),show_water=F,
      main=paste("Dynamic / Constant climate",yy[n],vars[k]),legend_colours=colo)
    dev.off()
  }
}

