# calculate global means for different runs with and without nitrogen
# SR 6.1.2016

#lpj_ref2 variante ohne wasserfix durch image leute
# lpj_ref variante mit fix

library(ncdf4)
source("read_LPJmL_nc.R")
oneplot <- TRUE
version <- 3
pathlim <- c("/p/projects/biodiversity/bloh/nitrogen/rcp85_cc/output_008/",
  "/p/projects/biodiversity/bloh/nitrogen/rcp85/output/",
  "/p/projects/biodiversity/bloh/nitrogen/output/")[version]

pathnon <- c("/p/projects/biodiversity/bloh/nitrogen/rcp85_cc/output/",
  "/p/projects/biodiversity/bloh/nitrogen/rcp85/output/",
  NA)[version]

kl <- c("08cc","12","ww")[version]

vars <- c("mnpp","mgpp","mrh","vegc","soilc")
filx <- c(".nc","_non.nc")

y1 <- 1901
y2 <- 2099
ys <- rep(y1:y2,each=12)

mach_ts <- function(file,y1=1901){
  mm <- read_LPJmL_nc_meta(file)
  area <- (111.263*0.5)*(111.263*0.5)*cos(mm$meta$latitude*pi/180.)
  marea <- matrix(rep(area,each=dim(mm$data)[1]),ncol=length(area),byrow=T)
  ii <- which(is.na(mm$data[,,1]))
  marea[ii] <- NA
  for (i in 1:dim(mm$data)[3]) mm$data[,,i] <- mm$data[,,i]*marea
  ms <- apply(mm$data,3,sum,na.rm=T)/sum(marea,na.rm=T)
  if (mm$meta$time[1] < 1900){
    years <- as.POSIXct(paste(y1,1,1,sep="-")) + mm$meta$time*3600*24
    years <- as.numeric(format(years,"%Y"))
    yy <- aggregate(ms,list(year=years),sum)$x
  } else yy <- ms
  yy
}

res_ts <- NULL
for (v in vars){
  print(c(v,filx[1]))
  d1 <- mach_ts(paste0(pathlim,v,filx[1]))
  print(c(v,filx[2]))
  d2 <- mach_ts(paste0(pathnon,v,filx[2]))
  res_ts <- c(res_ts,list(cbind(d1,d2)))
}
names(res_ts) <- vars
if (all(c("mnpp","mgpp") %in% vars)){
  res_ts <- c(res_ts,list(res_ts$mgpp - res_ts$mnpp))
  names(res_ts)[length(res_ts)] <- "mra"
}

save(res_ts,file=paste0("res_ts_",kl,".RData"))

### plots
if (oneplot) {
  pdf(paste0("allplots_",kl,".pdf"))
  par(las=1,lwd=2)
}
for (v in names(res_ts)){
  # ratio
  if (!oneplot) {
    pdf(paste0(v,"_",kl,".pdf"))
    par(las=1,lwd=2)
  }
  plot(y1:y2,res_ts[[v]][,1]/res_ts[[v]][,2],type="l",ylab="N limited / nonlimited",xlab="",
    main=paste0(v,"  kl=0.",kl))
  if (!oneplot) dev.off()
  # timeseries
  if (!oneplot) {
    pdf(paste0(v,"_series_",kl,".pdf"))
    par(las=1,lwd=2)
  }
  matplot(y1:y2,res_ts[[v]],type="l",lty=1,ylab="",xlab="",col=1:2,main=paste0(v,"  kl=0.",kl))
  legend("right",c("N limited","nonlimited"),lty=1,lwd=2,col=1:2,bty="n")
  if (!oneplot) dev.off()
}

allrat <- NULL
for (v in names(res_ts)){
  ratio <- res_ts[[v]][,1]/res_ts[[v]][,2]
  allrat <- cbind(allrat,ratio)
}
allrat <- as.data.frame(allrat)
names(allrat) <- names(res_ts)
if (!oneplot) {
  pdf(paste0("all_",kl,".pdf"))
  par(las=1,lwd=2)
}
matplot(y1:y2,allrat,type="l",lty=1,col=1:ncol(allrat),ylab="N limited / nonlimited",xlab="",
    main=paste0("kl=0.",kl))
legend("right",names(allrat),lty=1,lwd=2,col=1:ncol(allrat),bty="n")
if (!oneplot) dev.off()

if (oneplot) dev.off()


