#-------------------------------------------------------------------
# cal_helper.r
# R-Script prepare LPJ simulations with fixed LAIMAX 1 ... 7 for crop calibration,
# prepare calibration script, and construct new laimax parameter file.
#
# Tim Beringer
# 18.12.2013
#-------------------------------------------------------------------


# calibration workflow ####

# 1. check out trunk from SVN server
# 2. compile code
# 3. use this script to prepare lpj simulations with laimax 1 to 7
# 4. use lpjdistribute.sh to start simulations
# 5. call calibration script
# 6. create new laimax.par file with this script

closeAllConnections()
rm(list=ls(all=TRUE))
gc()

# SET DIRECTORIES, EMAIL, IDENTITY, ETC. ####

  # path to source code from computer running this script
  d.base <- c("/iplex/01/")
  # path to source code on cluster, initial dir in jcf file
  d.host <- "/iplex/01/"
  # directory used by calibration script, will be created if not existing
  d.cal <- c(paste(d.base,"output.yields/",sep=""))
  d.par <- c(paste(d.base,"par/",sep=""))
  
  # just in case the lpjml executable has been renamed
  f.lpj.bin <- "lpjml"
  lpj.bin <- paste(d.host,"bin/",f.lpj.bin,sep="")
  
  # just in case standard output directory in lpjml.conf hast already been changed
  lpjml.conf.out.dir <- "output_lu/"
  
  # set email for job notification
  my.email <- "me@pik-potsdam.de"
  my.name <- "My Name"
  
  # define files for output
  f.out <- c("------------------- --- ----------------------------- */",
             "GRID                RAW output/grid.bin",
             "FPC                 RAW output/fpc.bin ",
             "MNPP                RAW output/mnpp.bin",
             "MGPP                RAW output/mgpp.bin",
             "MRH                 RAW output/mrh.bin",
             "MFPAR               RAW output/mfpar.bin",
             "MTRANSP             RAW output/mtransp.bin",
             "MRUNOFF             RAW output/mrunoff.bin",
             "MEVAP               RAW output/mevap.bin",
             "MINTERC             RAW output/minterc.bin",
             "MSWC1               RAW output/mswc1.bin",
             "MSWC2               RAW output/mswc2.bin",
             "FIREC               RAW output/firec.bin",
             "FIREF               RAW output/firef.bin",
             "VEGC                RAW output/vegc.bin",
             "SOILC               RAW output/soilc.bin",
             "LITC                RAW output/litc.bin",
             "FLUX_ESTAB          RAW output/flux_estab.bin",
             "#ifdef WITH_SPITFIRE",
             "MFIREC              RAW output/mfirec.bin",
             "MNFIRE              RAW output/mnfire.bin",
             "MBURNTAREA          RAW output/mburnt_area.bin",
             "#endif",
             "#ifdef RIVER_ROUTING",
             "MDISCHARGE          RAW output/mdischarge.bin",
             "#endif",
             "#ifdef WITH_LANDUSE",
             "HARVEST             RAW output/flux_harvest.bin",
             "SDATE               RAW output/sdate.bin",
             "PFT_HARVEST         RAW output/pft_harvest.SUFFIX",
             "CFTFRAC             RAW output/cftfrac.bin",
             "#endif",
             "/*----------------- --- ------------------------------- */")
  
  f.jcf.ref <- c(
    "#!/bin/ksh",
    "# @ job_type = parallel",
    "# @ total_tasks = 32",
    "# @ class = short",
    "# @ group = elis",
    "# @ comment = LPJmL Version 3.5.003 args: -DFROM_RESTART",
    "# @ environment = COPY_ALL",
    "# @ blocking = UNLIMITED",
    "# @ output = /lpjml.out",
    "# @ error = /lpjml.err",
    "# @ notify_user = me@pik-potsdam.de",
    "# @ notification = complete",
    "# @ initialdir = /",
    "# @ queue",
    "",
    "llgetmachinelist > hostlist.$LOADL_STEP_ID",
    "",
    "cat hostlist.$LOADL_STEP_ID|uniq >mpdhosts.$LOADL_STEP_ID",
    "machine_count=`cat mpdhosts.$LOADL_STEP_ID  | wc -l`",
    "node_count=`cat hostlist.$LOADL_STEP_ID  | wc -l`",
    "export MPD_CON_EXT=$LOADL_STEP_ID",
    "mpdboot -n $machine_count -r ssh -f mpdhosts.$LOADL_STEP_ID",
    "",
    "mpiexec -machinefile hostlist.$LOADL_STEP_ID -n $node_count  /bin/lpjml -DFROM_RESTART lpjml.conf",
    "rc=$?  # save return code of mpiexec",
    "#mpdallexit",
    "rm hostlist.$LOADL_STEP_ID",
    "rm mpdhosts.$LOADL_STEP_ID",
    "exit $rc # exit with return code")
  
  f.param.par <- readLines(paste(d.par,"param.par",sep=""))
  pos.param.par <- grep("maximum LAI",f.param.par)
  f.param.conf <- readLines(paste(d.base,"param.conf",sep=""))
  pos.param.conf <- grep("LPJ parameter file",f.param.conf)
  f.lpjml.conf <- readLines(paste(d.base,"lpjml.conf",sep=""))
  pos1.lpjml.conf <- grep("BENCHMARK",f.lpjml.conf)
  pos2.lpjml.conf <- grep("Input parameter file",f.lpjml.conf)

# edit jcf file ####

  # change email for notification
    f.jcf.ref[grep("notify_user",f.jcf.ref)] <- 
      paste(unlist(strsplit(f.jcf.ref[grep("notify_user",f.jcf.ref)],"="))[1],"= ", my.email, sep="")
  # add LPJROOT definition
    f.jcf.ref <- c(f.jcf.ref[1:15],paste("export LPJROOT=",d.host,sep=""),"",f.jcf.ref[16:length(f.jcf.ref)])

# edit lpjml.conf ####

  # define BENCHMARK_LAI
    f.lpjml.conf[grep("#define BENCHMARK_LAI",f.lpjml.conf)] <- "#define BENCHMARK_LAI"  
  # define WITH_LANDUSE
    f.lpjml.conf[grep(glob2rx("*#define *LANDUSE*"),f.lpjml.conf)[1]] <- 
      "#define WITH_LANDUSE 1 /*NO_LANDUSE: land use disabled; WITH_LANDUSE: land use enabled */"
  # set spinup years
    f.lpjml.conf[grep("spinup years",f.lpjml.conf)[3]] <-
      "120 /* spinup years */"
  # set first year of simulation
    f.lpjml.conf[grep("first year of simulation",f.lpjml.conf)[2]] <-
      "1901 /* first year of simulation */"
  # set last year of simulation
    f.lpjml.conf[grep("last year of simulation",f.lpjml.conf)[3]] <-
      "2009 /* last year of simulation */"
  # do not start from restart
    f.lpjml.conf[grep("start from restart file",f.lpjml.conf)[2]] <-
      "NO_RESTART /* start from restart file */"
  # uncomment restart filename
    f.lpjml.conf[grep("filename of restart file",f.lpjml.conf)[2]] <-
      "/* restart/restart.lpj.dummy*/ /* filename of restart file */"
  # create restart file in 1900
    f.lpjml.conf[grep("create restart file",f.lpjml.conf)[2]] <-
      "RESTART /* create restart file */"
  # set year of restart
    f.lpjml.conf[grep("write restart at year",f.lpjml.conf)[1]] <-
      "1900 /* write restart at year; exclude line in case of no restart to be written */"
  
  param.conf.praef <- substr(f.param.conf[pos.param.conf],1,20)
  param.conf.suf <- substr(f.param.conf[pos.param.conf],20,nchar(f.param.conf[pos.param.conf]))
  
  lpjml.conf.praef <- substr(f.lpjml.conf[pos2.lpjml.conf],1,16)
  lpjml.conf.suf <- substr(f.lpjml.conf[pos2.lpjml.conf],16,nchar(f.lpjml.conf[pos2.lpjml.conf]))

# create lpjml.conf, param.conf, param.par for 7 LAIMAX simulations ####

  for(i in 1:7){
    # create param.par.laimax.*.par
    substr(f.param.par[pos.param.par],1,1) <- as.character(i)
    writeLines(f.param.par,paste(d.par,"param.laimax.",i,".par",sep=""))
    
    # create param.laimax.*.conf
    f.param.conf[pos.param.conf] <- paste(param.conf.praef,"laimax.",i,param.conf.suf,sep="")
    writeLines(f.param.conf,paste(d.base,"param.laimax.",i,".conf",sep=""))
    
    # create lpjml.laimax.*.conf
    f.lpjml.conf.laimax <- gsub(lpjml.conf.out.dir, paste("output.laimax.",as.character(i),"/",sep=""), f.lpjml.conf)
    f.lpjml.conf.laimax[grep("Input parameter file",f.lpjml.conf.laimax)] <-
      paste(lpjml.conf.praef,"laimax.",i,lpjml.conf.suf,sep="")
    # change output directory
    f.lpjml.conf.laimax[grep("output",f.lpjml.conf.laimax)]
    for(j in 1:length(grep("RAW",f.lpjml.conf.laimax))){
      unlist(strsplit(f.lpjml.conf.laimax[grep("RAW",f.lpjml.conf.laimax)[j]],"/"))
    }
    
    # change output directory
    pos.start <- grep(glob2rx(f.out[1]),f.lpjml.conf.laimax) # find start of output file section
    pos.end <- grep(glob2rx(f.out[34]),f.lpjml.conf.laimax) # find end of output file section
    length.output <- pos.end - pos.start + 1
    if(length.output != length(f.out)) print("WARNING: Check definition of output files in lpjml*.conf!")
    
    for(j in (pos.start+1):(pos.end-1)){
      if(length(unlist(strsplit(f.out[j-pos.start+1],"output/")))==2)
        f.lpjml.conf.laimax[j] <- 
          paste(unlist(strsplit(f.out[j-pos.start+1],"output/"))[1],"output.laimax.",i,"/",unlist(strsplit(f.out[j-pos.start+1],"output/"))[2],sep="")
    }
    
    # change name of restart file to be written after 1900
    f.lpjml.conf.laimax[grep("filename of restart file",f.lpjml.conf.laimax)[3]] <- 
      paste("restart/restart_1900_laimax.",i,".lpj /* filename of restart file */",sep="")
    writeLines(f.lpjml.conf.laimax,paste(d.base,"lpjml.laimax.",i,".conf",sep=""))
    
    # create lpjml.laimax.*.jcf
    lpj.conf.laimax <- paste(d.host,"lpjml.laimax.",i,".conf",sep="")
    f.jcf.ref[grep("mpiexec -machinefile",f.jcf.ref)]  <-
      paste("mpiexec -machinefile hostlist.$LOADL_STEP_ID -n $node_count ", lpj.bin, " -DFROM_RESTART", lpj.conf.laimax)
    f.jcf.ref[grep("# @ output",f.jcf.ref)] <- 
      paste(unlist(strsplit(f.jcf.ref[grep("# @ output",f.jcf.ref)],"="))[1]," = ", d.host,"output.laimax.",i,"/lpjml.out",sep="")
    f.jcf.ref[grep("# @ error",f.jcf.ref)] <- 
      paste(unlist(strsplit(f.jcf.ref[grep("# @ error",f.jcf.ref)],"="))[1]," = ", d.host,"output.laimax.",i,"/lpjml.err",sep="")
    f.jcf.ref[grep("# @ initialdir",f.jcf.ref)] <- 
      paste(unlist(strsplit(f.jcf.ref[grep("# @ initialdir",f.jcf.ref)],"="))[1]," = ", d.host, sep="")
    
    writeLines(f.jcf.ref,paste(d.base,"lpjml.laimax.",i,".jcf",sep=""))
    
    # create output directories
    dir.create(paste(d.base,"output.laimax.",i,sep=""))
  }

# RUN LPJ SIMULATIONS ####

for(laimax ...)
system()

if(!file.exists(paste(d.base,"output.yields",sep=""))) dir.create(paste(d.base,"output.yields",sep=""))
setwd(paste(d.base,"output.yields/",sep=""))

  for(i in 1:7){
    if(!file.exists(paste(d.base,"output.yields/","cftfrac.LAI",i,".bin",sep="")))
      file.symlink(paste(d.base,"output.laimax.",i,"/cftfrac.bin",sep=""),paste(d.base,"output.yields/","cftfrac.LAI",i,".bin",sep=""))
    
    if(!file.exists(paste(d.base,"output.yields/","pft_harvest.LAI",i,".bin",sep="")))
      file.symlink(paste(d.base,"output.laimax.",i,"/pft_harvest.pft.bin",sep=""),paste(d.base,"output.yields/","pft_harvest.LAI",i,".pft.bin",sep=""))
  }
  if(!file.exists(paste(d.base,"output.yields/grid.bin",sep="")))
    file.symlink(paste(d.base,"output.laimax.1/grid.bin",sep=""),paste(d.base,"output.yields/grid.bin",sep=""))

# PREPARE AND RUN CALIBRATION SCRIPT ####

  y.first <- 1901
  y.start <- 1999
  y.end <- 2003
  p.masks <- "/data/biosx/LPJ/calib/" # place where cal script looks for lpjLandis0_mask.img and countryMask.img
  p.input <- "/iplex/01/2012/open/input_VERSION2/" # directory where cal script looks for input grid.bin and cow_mg_2006.bin
  p.output <- "/iplex/01/2012/open/Tim/test_calib_1/output.yields/" # directory where cal script looks for results from LAIMAX 1-7 simulations
  h.size.grid <- 43 # size of input data header in bytes, old version has size 35
  h.size.country <- 43 # size of input data header in bytes, old version has size 39

  # add part that changes paths, etc. in calibration script
  f.cal <- readLines(paste(d.base,"R/global_fao_vs_lpjml_calib.r",sep=""))
  f.cal[grep("# no of cells recorded in file",f.cal)] <- "ncell <- 67420 # no of cells recorded in file"
  f.cal[grep("#first year to evaluate",f.cal)] <- paste("startyear <- ",y.start-y.first+1, " #first year to evaluate",sep="")
  f.cal[grep("#last year to evaluate",f.cal)] <- paste("endyear <- ",y.end-y.first+1, " #last year to evaluate",sep="")
  f.cal[grep("fao.years",f.cal)[1]] <- paste("fao.years <- c(",y.start,":",y.end,")",sep="")
  f.cal[grep("# set script working path",f.cal)+1] <- paste("script.path <- \"",p.masks,"\"",sep="")
  f.cal[grep("lpjinput.path <- ",f.cal)] <- paste("lpjinput.path <- \"",p.input,"\"",sep="")
  f.cal[grep("lpjoutput.path <- ",f.cal)] <- paste("lpjoutput.path <- \"",p.output,"\"",sep="")
  f.cal[grep(glob2rx("seek(file.grid.input*"),f.cal)] <- paste("seek(file.grid.input,where=",h.size.grid,",start=\"origin\")",sep="")
  f.cal[grep(glob2rx("seek(file.country.input*"),f.cal)] <- paste("seek(file.country.input,where=",h.size.country,",start=\"origin\")",sep="")

  writeLines(f.cal,paste(paste(d.base,"R/global_fao_vs_lpjml_calib.r",sep="")))

  # run calibration script ####
  source(paste(d.base,"R/global_fao_vs_lpjml_calib.r",sep=""))
    
# CREATE MODIFIED PARAM_LAIMAX.PAR ####

  f.param.laimax.par.h <- readLines(paste(d.par,"manage_laimax_alphaa_revision_1241_sc.par",sep=""),n=16)
  f.param.laimax.par <- read.table(paste(d.par,"manage_laimax_alphaa_revision_1241_sc.par",sep=""),skip=16)
  
  f.laimax.new <- read.table(paste(d.cal,"copy_me_by_hand_to_manage_laimax_alphaa_revision_1200_sc.par.txt",sep=""))

  for(i in 1:197){
    f.param.laimax.par[i,3:14] <- f.laimax.new[i,]  
  }
  # rename 'Last change' in par header
  f.param.laimax.par.h[8]<-  paste("/**    Last change: ",format(Sys.Date(), "%d.%m.%Y"),", ",my.name,"                         **/\t\t\t")

  # add revision to file name, climate data to header, cal time frame
  rev <- system(paste("svnversion ",d.host,sep=""), intern = TRUE)
  
  paste(d.par,"manage_laimax_alphaa_fao.y.",y.start,".",y.end,"_date.",Sys.Date(),"_lpj.rev.",rev,".par",sep="")
  
  writeLines(f.param.laimax.par.h,paste(d.par,"manage_laimax_alphaa_fao.y.",y.start,".",y.end,"_date.",Sys.Date(),"_lpj.rev.",rev,".par",sep=""))
  write.table(f.param.laimax.par,paste(d.par,"manage_laimax_alphaa_fao.y.",y.start,".",y.end,"_date.",Sys.Date(),"_lpj.rev.",rev,".par",sep=""), append=T, sep = "\t", col.names=FALSE, row.names=FALSE, quote=c(2))
  
