
mach_ts <- function(path,files,y1=1901,n=NA,usefrac=NA){
  mmc <- read_LPJmL_nc_meta(paste0(path,files[1]),n=n)
  mmn <- read_LPJmL_nc_meta(paste0(path,files[2]),n=n)
  area <- (111.263*0.5)*(111.263*0.5)*cos(mmn$meta$latitude*pi/180.)
  marea <- matrix(rep(area,dim(mmn$data)[1]),ncol=length(area),byrow=T)
  aarea <- array(marea,dim=c(dim(marea),dim(mmc$data)[3]))
  if (usefrac & !is.na(n)){
    if (n<11){ #natural vegetation
      ff <- read_LPJmL_nc_meta(paste0(path,"fpc.nc"),n=n+1)
      frac <- ff$data
    } else {
      frac <- 0*aarea
      frac[,,1:dim(cftmat)[4]] <- cftmat[n-10,,,]
      if (dim(cftmat)[4] < dim(aarea)[3]) 
        frac[,,(dim(cftmat)[4]+1):dim(aarea)[3]] <- frac[,,dim(cftmat)[4]]
    }
    aarea <- aarea*frac
  }
  
  mm <- mmc$data/mmn$data
  jj <- which(is.na(mm))
  aarea[jj] <- NA
  ii <- which(mm > 5000)
  mm[ii] <- NA
  aarea[ii] <- NA
  ma <- mm*aarea
  ms <- apply(ma,3,sum,na.rm=T)/apply(aarea,3,sum,na.rm=T)
  if (mmc$meta$time[1] < 1900){
    years <- as.POSIXct(paste(y1,1,1,sep="-")) + mmc$meta$time*3600*24
    years <- as.numeric(format(years,"%Y"))
    yy <- aggregate(ms,list(year=years),sum)$x
  } else yy <- ms
  if (any(is.infinite(yy))) {
    ii <- which(is.infinite(yy))
    yy[ii] <- NA
  }
  yy
}

read_LPJmL_bin <- function(filename,n=NA,y2=2009,y1=1901,ncell=67420,fpc=FALSE,month=FALSE,
  ry=NA,ay=NA,band=NA){
  # if n is specified, only band n is read, otherwise all bands
  # ry: year to be read, when not specified, reads all years
  # ay: when specified with ry, averaging ry-ay to ry+ay
  nf <- file(filename,"rb")
  # check dimensions
  yl <- y2-y1+1
  sf <- file.size(filename)
  it <- sf/ncell/yl
  sb <- ifelse(it/2 > 50,4,2)
  format <- numeric()
  if (fpc){
    if (!is.na(n)) n <- n+1
    sb <- 4
  }
  np <- it/sb
  if (month){
    np <- 12
    sb <- it/np
  }
  if (!is.na(band)){
    np <- band
    sb <- it/np
  }
  if (!is.na(n)){
    if (is.na(ry)){
      data <- array(NA,dim=c(ncell,yl))
      for (j in 1:yl){
        daty <- readBin(nf,format,n=ncell*np,size=sb)
        datm <- matrix(daty,ncol=np)
        data[,j] <- datm[,n]
      } 
    } else {
      if (is.na(ay)){
        seek(nf,(ry-y1)*np*sb*ncell)
        daty <- readBin(nf,format,n=ncell*np,size=sb)
        datm <- matrix(daty,ncol=np)
        data <- datm[,n]
      } else {
        data <- array(0,dim=c(ncell,np))
        seek(nf,(ry-ay-y1)*np*sb*ncell)
        for (j in 1:(2*ay+1)){
          daty <- readBin(nf,format,n=ncell*np,size=sb)
          datm <- matrix(daty,ncol=np)
          data <- data + datm
        }
        data <- data[,n]/(2*ay+1)
      }
    }
  } else {
    if (is.na(ry)){
      data <- array(NA,dim=c(ncell,yl,np))
      for (j in 1:yl) data[,j,] <- readBin(nf,format,n=ncell*np,size=sb)
    } else {
      if (is.na(ay)){
        seek(nf,(ry-y1)*np*sb*ncell)
        daty <- readBin(nf,format,n=ncell*np,size=sb)
        data <- matrix(daty,ncol=np)
      } else {
        data <- array(0,dim=c(ncell,np))
        seek(nf,(ry-ay-y1)*np*sb*ncell)
        for (j in 1:(2*ay+1)){
          daty <- readBin(nf,format,n=ncell*np,size=sb)
          datm <- matrix(daty,ncol=np)
          data <- data + datm
        }
        data <- data/(2*ay+1)
      }
    }
  }
  close(nf)
  list(data=data)
}

create_weighted_average <- function(x,y,vmax=cnmax){
  jj <- which(!is.finite(x))
  y[jj] <- NA
  ii <- which(x > vmax)
  x[ii] <- NA
  y[ii] <- NA
  xa <- x*y
  yy <- apply(xa,2,sum,na.rm=T)/apply(y,2,sum,na.rm=T)
  if (any(!is.finite(yy))) {
    ii <- which(!is.finite(yy))
    yy[ii] <- NA
  }
  yy
}

create_quantile <- function(x,vmax=cnmax,qq=0.95){
  jj <- which(!is.finite(x))
  x[jj] <- NA
  ii <- which(x > vmax)
  x[ii] <- NA
  yy <- apply(x,2,quantile,qq,na.rm=T)
  if (any(!is.finite(yy))) {
    ii <- which(!is.finite(yy))
    yy[ii] <- NA
  }
  yy
}

checknegative <- function(tab){
  ret <- NULL
  ii <- which(tab < 0)
  if (length(ii) > 0){
    dd <- dim(tab)
    ks <- rep(1:dd[1],dd[2])
    js <- rep(1:dd[2],each=dd[1])
    j <- js[ii]
    k <- ks[ii]
    # (m in 1:length(ii)) print(c(k[m],j[m],ii[m],tab[k[m],j[m]]))
    ret <- data.frame(year=rep(NA,length(k)),row=k,col=j,val=tab[ii])
  }
  ret
}

mach_ts_bin <- function(path,files,y1=1901,y2=2009,lp=1,usefrac=NA,cgrid=grid,cnmax=5000,kenn="cn_neg",pnv=TRUE){
  #mmc <- read_LPJmL_bin(paste0(path,files[1]),n=n)
  #mmn <- read_LPJmL_bin(paste0(path,files[2]),n=n)
  numcell <- nrow(cgrid)
  cfile <- paste0(path,files[1])
  nfile <- paste0(path,files[2])
  yl <- y2-y1+1               # number of years
  sf <- file.size(cfile)      # file size
  it <- sf/numcell/yl         # bytes * bands
  sb <- ifelse(it/2 > 50,4,2) # bytes per entry
  if (usefrac) sb <- 4
  np <- it/sb                 # number of bands
  format <- numeric()

  area <- (111.263*0.5)*(111.263*0.5)*cos(cgrid[,2]*pi/180.)
  aarea <- matrix(rep(area,yl),nrow=length(area))
  cf <- file(cfile,"rb")
  nf <- file(nfile,"rb")
  nn <- file(paste0(path,"fpc.bin"),"rb")

  data <- array(NA,dim=c(yl,np,5))
  negc <- negn <- NULL
  for (j in 1:yl){
    if (ceiling(j/10)*10 == j) print(j)
    daty <- readBin(cf,format,n=ncell*np,size=sb)
    cdat <- matrix(daty,ncol=np)
    cneg <- checknegative(cdat)
    if (!all(is.null(cneg))){
      ii <- which(cdat<0)
      cdat[ii] <- 0
      cneg$year <- j
      negc <- rbind(negc,cneg)
    }
    daty <- readBin(nf,format,n=ncell*np,size=sb)
    ndat <- matrix(daty,ncol=np)
    nneg <- checknegative(ndat)
    if (!all(is.null(nneg))){
      ii <- which(ndat<0)
      ndat[ii] <- 0
      nneg$year <- j
      negn <- rbind(negn,nneg)
    }
    if (usefrac){
      mat  <- matrix(readBin(nn,numeric(),n=numcell*11,size=4),nrow=numcell)
      frac <- array(NA,dim=c(numcell,np))
      if (pnv){
        nv <- mat[,1]
        versatz <- 10
        frac[,1:10] <- nv*mat[,-1]
      } else {
        nv <- mat[,1]
        versatz <- 0
      }
      if (j <= dim(cftfrac)[3]){
        frac[,versatz+1:dim(cftfrac)[1]] <- (1-nv)*t(cftfrac[,,j])
      } else {
        frac[,versatz+1:dim(cftfrac)[1]] <- (1-nv)*t(cftfrac[,,dim(cftfrac)[3]])
      }
      wicht <- area*frac
    } else {
      wicht <- area
    }
    data[j,,1] <- create_weighted_average(cdat,wicht,1e4)
    data[j,,2] <- create_weighted_average(ndat,wicht,1e4)
    data[j,,3] <- create_weighted_average(cdat/ndat,wicht,cnmax)
    data[j,,4] <- create_quantile(cdat/ndat,cnmax,0.10)
    data[j,,5] <- create_quantile(cdat/ndat,cnmax,0.90)
  }
  close(cf)
  close(nf)
  close(nn)
  save(negc,negn,file=paste0("negative_",kenn,".RData"))
  data
}


# mach_ts_bin(paths,pfiles,lp=lp,usefrac=usefpc)
mach_ave_bin <- function(path,files,y1=1901,y2=2009,lp=1,yw=2000,usefrac=NA,cgrid=grid,cnmax=5000,kenn="cn_neg",pnv=TRUE,yave=4){
  #mmc <- read_LPJmL_bin(paste0(path,files[1]),n=n)
  #mmn <- read_LPJmL_bin(paste0(path,files[2]),n=n)
  numcell <- nrow(cgrid)
  cfile <- paste0(path,files[1])
  nfile <- paste0(path,files[2])
  yl <- y2-y1+1               # number of years
  sf <- file.size(cfile)      # file size
  it <- sf/numcell/yl         # bytes * bands
  sb <- ifelse(it/2 > 50,4,2) # bytes per entry
  if (usefrac) sb <- 4
  np <- it/sb                 # number of bands
  format <- numeric()

  yr <- yw-y1+1 # year to read  
  yr1 <- yr-yave
  yr2 <- yr+yave

  area <- (111.263*0.5)*(111.263*0.5)*cos(cgrid[,2]*pi/180.)
  aarea <- matrix(rep(area,yl),nrow=length(area))
  cf <- file(cfile,"rb")
  nf <- file(nfile,"rb")
  nn <- file(paste0(path,"fpc.bin"),"rb")
  if (yr > 4){
    seek(cf, where = ncell*np*sb*(yr1-1), origin = "start")
    seek(nf, where = ncell*np*sb*(yr1-1), origin = "start")
    if (usefrac)
      seek(nn, where = numcell*11*4*(yr1-1), origin = "start")
  }

  data <- array(NA,dim=c(2*yave+1,np,5))
  negc <- negn <- NULL
  for (j in 1:(2*yave+1)){
    daty <- readBin(cf,format,n=ncell*np,size=sb)
    cdat <- matrix(daty,ncol=np)
    cneg <- checknegative(cdat)
    if (!all(is.null(cneg))){
      ii <- which(cdat<0)
      cdat[ii] <- 0
      cneg$year <- j
      negc <- rbind(negc,cneg)
    }
    daty <- readBin(nf,format,n=ncell*np,size=sb)
    ndat <- matrix(daty,ncol=np)
    nneg <- checknegative(ndat)
    if (!all(is.null(nneg))){
      ii <- which(ndat<0)
      ndat[ii] <- 0
      nneg$year <- j
      negn <- rbind(negn,nneg)
    }
    if (usefrac){
      mat  <- matrix(readBin(nn,numeric(),n=numcell*11,size=4),nrow=numcell)
      frac <- array(NA,dim=c(numcell,np))
      if (pnv){
        nv <- mat[,1]
        versatz <- 10
        frac[,1:10] <- nv*mat[,-1]
      } else {
        nv <- mat[,1]
        versatz <- 0
      }
      if (j <= dim(cftfrac)[3]){ 
        frac[,versatz+1:dim(cftfrac)[1]] <- (1-nv)*t(cftfrac[,,j])
      } else {
        frac[,versatz+1:dim(cftfrac)[1]] <- (1-nv)*t(cftfrac[,,dim(cftfrac)[3]])
      }
      wicht <- area*frac
    } else {
      wicht <- area
    }
    data[j,,1] <- create_weighted_average(cdat,wicht,1e4)
    data[j,,2] <- create_weighted_average(ndat,wicht,1e4)
    data[j,,3] <- create_weighted_average(cdat/ndat,wicht,cnmax)
    data[j,,4] <- create_quantile(cdat/ndat,cnmax,0.10)
    data[j,,5] <- create_quantile(cdat/ndat,cnmax,0.90)
  }    
  close(cf)
  close(nf)
  close(nn)
  save(negc,negn,file=paste0("negative_",kenn,".RData"))
  data
}

myplotmap2 <- function (data, file = NULL, title = "World map", legend_range = NULL,
                        legendname = "Cell share", colorscale = NULL, lowcol = "grey95", 
                        midcol = "orange", highcol = "darkred", midpoint = 0.5, 
                        facet_grid = "Year~Data1", scale = 2, breaks = T, labs = T, 
                        borders = TRUE, MAgPIE_regions = F, axis_text_col = "black", 
                        legend_discrete = FALSE, legend_breaks = NULL,
                        show_percent = FALSE, sea = TRUE, land_colour = "white",
                        legend_height = NULL, legend_width = NULL, text_size = 12,
                        legend_position = "right", facet_style = "default",...){
  require("ggplot2", quietly = TRUE)
  require("RColorBrewer", quietly = TRUE)
  wrld_simpl_df <- NULL
  data("world", envir = environment(), package = "luplot")

  if (MAgPIE_regions) {
    facet_grid <- NULL
    map <- ggplot(wrld_simpl_df, aes_(~long, ~lat)) + 
      geom_polygon(aes_(group = ~group, fill = ~magpie)) + 
      scale_fill_manual("MAgPIE\nregion", 
                        values = c("purple3", "red3", "hotpink", "cyan3",
                                   "goldenrod1", "gray44", "#8C5400FF", "darkorange2",
                                   "royalblue3", "green4"), na.value = "white")
  } else {
    if (!is.list(data)) {
      temp <- data
      data <- list()
      data[["default"]] <- temp
    }
    if (is.null(legend_range)) {
      midpoint <- max(unlist(lapply(data, max, na.rm = T))) * midpoint
    } else {
      data <- lapply(data, function(x) {
        x[which(x < legend_range[1])] <- legend_range[1]
        x[which(x > legend_range[2])] <- legend_range[2]
        return(x)
      })
    }
    if (any(unlist(lapply(data, function(x) return(is.null(attr(x,"coordinates"))))))) {
      data <- lapply(data, function(x) {
        attr(x, "coordinates") <- getCoordinates(degree = T)
        return(x)
      })
      warning("Missing coordinates in attributes for at least one MAgPIE object. Added coordinates in default MAgPIE cell order.")
    }
    data <- as.ggplot(data, asDate = F)
    if (legend_discrete) {
      if (is.null(legend_breaks)) {
        data$Breaks <- as.character(data$Value)
        if (any(is.na(data$Breaks))) {
          data$Breaks[is.na(data$Breaks)] <- "No data"
        }
        data$Breaks <- as.factor(data$Breaks)
        legend_labels <- levels(data$Breaks)
      } else {
        tmp <- as.vector(data$Value)
        tmp[] <- length(legend_breaks) + 1
        legend_labels <- rep("", length(legend_breaks) + 1)
        legend_labels[length(legend_breaks) + 1] <- 
          paste(">", legend_breaks[length(legend_breaks)])
        for (i in length(legend_breaks):2) {
          tmp[which(as.vector(data$Value) <= legend_breaks[i])] <- i
          legend_labels[i] <- paste(legend_breaks[i-1], "-", legend_breaks[i])
        }
        tmp[which(as.vector(data$Value) < legend_breaks[1])] <- 1
        tmp[which(is.na(as.vector(data$Value)))] <- NA
        legend_labels[1] <- paste("<", legend_breaks[1])
        legend_labels <- legend_labels[as.numeric(rev(levels(as.factor(tmp))))]
        tmpchar <- as.character(tmp)
        tmpchar[is.na(tmpchar)] <- "No data"
        levels <- rev(levels(as.factor(tmp)))
        if ("No data" %in% tmpchar) {
          legend_labels <- c(legend_labels, "No data")
          levels <- c(levels, "No data")
        }
        data$Breaks <- factor(tmpchar, levels = levels)
      }
      if ("No data" %in% legend_labels) {
        colours <- c(colorRampPalette(c(highcol, midcol,
                                        lowcol))(length(legend_labels) - 1), "grey")
      } else {
        if (is.null(colorscale))
          colours <- colorRampPalette(c(highcol, midcol,
                                        lowcol))(length(legend_labels))
        else {
          colours <- colorRampPalette(brewer.pal(9,colorscale))(length(legend_labels))
        } 
      }
      if (show_percent) {
        tmp <- table(data$Breaks)
        if ("No data" %in% names(tmp))
          tmp <- tmp[-which(names(tmp) == "No data")]
        percent <- round(tmp/sum(tmp) * 100, 1)
        percent <- paste("(", percent, "%)", sep = "")
        legend_labels[which(legend_labels != "No data")] <- 
          paste(legend_labels[which(legend_labels !="No data")], percent, sep = "     ")
      }
    }
    if (!is.null(legend_breaks)) {
      labels <- c(bquote("" <= .(head(legend_breaks, 1))),
                  legend_breaks[2:(length(legend_breaks) - 1)],
                  bquote("" >= .(tail(legend_breaks, 1))))
    }
    else {
      legend_breaks <- waiver()
      labels <- waiver()
    }
    map <- ggplot(data, aes_(~x, ~y)) + 
      geom_polygon(data = wrld_simpl_df,aes_(~long, ~lat, group = ~group, fill = ~hole),
                   fill = land_colour)
    if (is.null(data$Breaks)) {
      if (!is.null(colorscale)){
        farb <- colorRampPalette(brewer.pal(9,colorscale))
        map <- map + geom_raster(aes_(fill = ~Value)) +
          scale_fill_gradientn(name = legendname, colours=farb(30),
                               limits = legend_range, breaks = legend_breaks,
                               labels = labels, na.value = "grey")
      
      } else if (!is.null(midcol))
        map <- map + geom_raster(aes_(fill = ~Value)) +
          scale_fill_gradient2(name = legendname, low = lowcol,
                               mid = midcol, high = highcol, midpoint = midpoint,
                               limits = legend_range, breaks = legend_breaks,
                               labels = labels, na.value = "grey")
      else map <- map + geom_raster(aes_(fill = ~Value)) +
          scale_fill_gradient(name = legendname, low = lowcol,
                              high = highcol, limits = legend_range, breaks = legend_breaks,
                              labels = labels, na.value = "grey")
    }
    else map <- map + geom_raster(aes_(fill = ~Breaks)) +
      scale_fill_manual(name = legendname, values = colours,
                        labels = legend_labels, na.value = "yellow")
    if (!is.null(legend_height))
      map <- map + theme(legend.key.height = unit(legend_height,"cm"))
    if (!is.null(legend_width))
      map <- map + theme(legend.key.width = unit(legend_width,"cm"))
  }
  map <- map + coord_cartesian(xlim = c(-180, 180), ylim = c(-58,86)) + 
    theme(aspect.ratio = 0.5) + ggtitle(title)
  if (sea) map <- map + theme(panel.background = element_rect(fill = "lightsteelblue2"))
  else map <- map + theme(panel.background = element_rect(fill = "white",colour = "black")) #+ 
    #theme(panel.grid.major = element_line(colour = "grey80"),
    #      panel.grid.minor = element_line(colour = "grey90"))
  if (!is.null(facet_grid))
    map <- map + facet_grid(facet_grid)
  if (breaks)
    map <- map + scale_x_continuous(breaks = c(-90, 0, 90)) +
    scale_y_continuous(breaks = c(-66, -38, -23, 0, 23,38, 66))
  else map <- map + scale_x_continuous(breaks = NULL) + scale_y_continuous(breaks = NULL)
  if (labs)
    map <- map + labs(x = "Longitude", y = "Latitude")
  else map <- map + labs(y = NULL, x = NULL)
  if (borders)
    map <- map + geom_path(data = wrld_simpl_df, 
                           aes_(~long,~lat, group = ~group, fill = NULL), 
                           color = "grey10",size = 0.1)
  if (!is.null(axis_text_col))
    map <- map + theme(axis.text = element_text(colour = axis_text_col),
                       axis.ticks = element_line(colour = axis_text_col))
  map <- map + theme(panel.grid.minor = element_line(colour = "white"),
                     plot.title = element_text(size = text_size+4,face = "bold",vjust = 1.5), 
                     legend.position = legend_position,
                     legend.title = element_text(size = text_size, face = "bold"),
                     legend.text = element_text(size = text_size), 
                     axis.title.y = element_text(angle = 90,size = text_size, vjust = 0.3), 
                     axis.text.y = element_text(size = text_size -2), 
                     axis.title.x = element_text(size = text_size,vjust = -0.3), 
                     axis.text.x = element_text(size = text_size -2, vjust = 0.5))
  if (facet_style == "default")
    map <- map + theme(strip.text.x = element_text(size = text_size - 1), 
                       strip.text.y = element_text(size = text_size - 1))
  else if (facet_style == "paper")
    map <- map + theme(strip.text.x = element_text(size = text_size, face = "bold"), 
                       strip.text.y = element_text(size = text_size, face = "bold")) + 
    theme(strip.background = element_blank())
  if (legend_position %in% c("top", "bottom"))
    map <- map + guides(fill = guide_colorbar(title.position = "top")) + 
    theme(legend.box.just = "left")
  if (!is.null(file)) {
    ggsave(file, map, scale = scale,...)
  }
  else {
    return(map)
  }
}


