# deletes all variables
rm(list=ls(all=TRUE))

# load required libaries
require(fields)
require(ncdf4)
require(maps)

# settings
ncell <- 67420
NDAYMONTH <- c(31,28,31,30,31,30,31,31,30,31,30,31)
data.path <- "/p/projects/landuse/users/cmueller/GGCMI/fertilizer_harm_1p1/"
lpj.path <- "/p/projects/isimip/isimip-lpj/input_CLM2/"

cropnames <- c("wheat","rice","maize","millet","NA","sugarbeet","cassava",
               "sunflower","soybean","groundnut","rapeseed","sugarcane","NA","NA","NA","NA")

################
# functions ####
################
read.LPJmL.grid <- function(filename,npix,header,swap=F){
  ff <- file(filename,"rb")
  seek(ff,where=header,origin="start")
  if(swap){
    x <- readBin(ff,integer(),size=2,n=npix*2,endian="swap")/100
  } else {
    x <- readBin(ff,integer(),size=2,n=npix*2)/100
  }
  lon <- x[(1:npix)*2-1]
  lat <- x[(1:npix)*2]
  # ilon/ilat are row and column numbers, starting at 179.75W and 89.75N as 1/1
  ilon <- as.integer((lon+180)/0.5 + 1.01)
  ilat <- as.integer((-lat+90)/0.5 + 1.01)
  ilatd <- as.integer((lat+90)/0.5 + 1.01)
  area <- (111e3*0.5)*(111e3*0.5)*cos(lat/180*pi)/10000
  close(ff)
  data.frame(lon,lat,ilon,ilat,ilatd,area)
}
fwriteheader <- function(file.out,headername,bands,scalar,fy=1,ny=1){
  writeChar(headername,file.out,eos=NULL)
  writeBin(as.integer(2),file.out,size=4,endian=.Platform$endian)     # CLIMATE VERSION
  writeBin(as.integer(1),file.out,size=4,endian=.Platform$endian)   # ORDER
  writeBin(as.integer(fy),file.out,size=4,endian=.Platform$endian) 	# FIRSTYEAR  
  writeBin(as.integer(ny),file.out,size=4,endian=.Platform$endian) 	# NYEAR      
  writeBin(as.integer(0),file.out,size=4,endian=.Platform$endian) 	# FIRSTCELL
  writeBin(as.integer(ncell),file.out,size=4,endian=.Platform$endian) 	# NCELL
  writeBin(as.integer(bands),file.out,size=4,endian=.Platform$endian) 	# BAND
  writeBin(0.5,file.out,size=4,endian=.Platform$endian) 	# CELLSIZE
  writeBin(scalar,file.out,size=4,endian=.Platform$endian) 	# SCALAR
}

#################
# main       ####
#################

grid <- read.LPJmL.grid(paste(lpj.path,"grid.bin",sep=""),ncell,43)
if(F){
data <- array(0,dim=c(720,360,length(cropnames)))
for(i in 1:length(cropnames)){
  if(cropnames[i]=="NA"){
    cat("skipping",i,"\n")
  } else{
    ncf <- nc_open(paste(data.path,"agmip_",cropnames[i],"_apprate_fill_NPK_0.5.nc4",sep=""))
    data[,,i] <- ncvar_get(ncf,"Napprate")
    nc_close(ncf)    
  }
}
vec <- vector(mode="integer",length=ncell*length(cropnames)*2)
# scale with 10 to keep first decimal place
for(p in 1:ncell){
  vec[c(((p-1)*32+1):(p*32))] <- as.integer(c(data[grid$ilon[p],grid$ilat[p],],data[grid$ilon[p],grid$ilat[p],])*10+.5)
}

fo <- file(paste(lpj.path,"fertilizer_ggcmi.clm2",sep=""),"wb")
#write header
fwriteheader(fo,"LPJFERT",32,0.01,fy=2000,ny=1) # scaler to 0.01 to convert from kg/ha to g/m2 and adjust for the factor 10 above (decimal place)
#write data
writeBin(vec,fo,size=2)
#close file
close(fo)
}

# read historic data file, use the 2000s from RCP settings
data <- array(0,dim=c(720,360,12,15,2))
for(i in 1:15){
  ncf <- nc_open(paste(lpj.path,"Data_03/ndep_ocn_",1850+(i-1)*10,"-",1859+(i-1)*10,"_0.5x0.5.nc",sep=""))
  data[,,,i,1] <- ncvar_get(ncf,"NOy_dry_deposition") + ncvar_get(ncf,"NOy_wet_deposition")
  data[,,,i,2] <- ncvar_get(ncf,"NHx_dry_deposition") + ncvar_get(ncf,"NHx_wet_deposition")
  nc_close(ncf)
}
# loop through RCPs
rcps <- c("26","45","60","85")
rcs <- c("2p6","4p5","6p0","8p5")
for(r in 1:length(rcps)){
  # read RCP data
  data2 <- array(0,dim=c(720,360,12,11,2))
  for(i in 1:11){
    ncf <- nc_open(paste(lpj.path,"Data_RCP",rcps[r],"/ndep_ocn_",2000+(i-1)*10,"-",2009+(i-1)*10,"_0.5x0.5.nc",sep=""))
    data2[,,,i,1] <- ncvar_get(ncf,"NOy_dry_deposition") + ncvar_get(ncf,"NOy_wet_deposition")
    data2[,,,i,2] <- ncvar_get(ncf,"NHx_dry_deposition") + ncvar_get(ncf,"NHx_wet_deposition")
    nc_close(ncf)  
  }

  # sort to vector
  no3dep <- vector(mode="integer",length=c(ncell*260*12))
  nh4dep <- vector(mode="integer",length=c(ncell*260*12))
  data3 <- array(0,dim=c(ncell,12,260,2))
  for(p in 1:ncell){
    if(p%%1000==0) cat("doing",p,"\n")
    # values work for 10 years each
    for(i in 1:15) data3[p,,c(1:10)+(i-1)*10,] <- rep(data[grid$ilon[p],grid$ilatd[p],,i,],10)
    for(i in 1:11) data3[p,,c(1:10)+(i-1+15)*10,] <- rep(data2[grid$ilon[p],grid$ilatd[p],,i,],10)
  }
  for(i in 1:260){
    # write data row by row
    no3dep[c(1:(ncell*12))+(i-1)*ncell*12] <- as.integer(as.vector(t(data3[,,i,1]))*1e5+0.5)
    nh4dep[c(1:(ncell*12))+(i-1)*ncell*12] <- as.integer(as.vector(t(data3[,,i,2]))*1e5+0.5)
  }
 
if(T){ 
  fo <- file(paste(lpj.path,"no3_deposition_rcp",rcs[r],"_int1e5.clm",sep=""),"wb")
  #write header
  fwriteheader(fo,"LPJNDEP",12,1e-5,fy=1850,ny=260)
  #write data
  writeBin(no3dep,fo,size=2)
  #close file
  close(fo)
  fo <- file(paste(lpj.path,"nh4_deposition_rcp",rcs[r],"_int1e5.clm",sep=""),"wb")
  #write header
  fwriteheader(fo,"LPJNDEP",12,1e-5,fy=1850,ny=260)
  #write data
  writeBin(nh4dep,fo,size=2)
  #close file
  close(fo)
}

# write zero-depostion files for GGCMI CTWN-A
fo <- file(paste(lpj.path,"zero_deposition","int1e5.clm",sep=""),"wb")
#write header
fwriteheader(fo,"LPJNDEP",12,1,fy=1850,ny=260)
#write data
no3dep[] <-as.integer(0)
writeBin(no3dep,fo,size=2)
#close file
close(fo)

  # do the same for float input
  # sort to vector
  no3dep <- vector(mode="numeric",length=c(ncell*260*12))
  nh4dep <- vector(mode="numeric",length=c(ncell*260*12))
  data3 <- array(0,dim=c(ncell,12,260,2))
  for(p in 1:ncell){
    if(p%%1000==0) cat("doing",p,"\n")
    # values work for 10 years each
    for(i in 1:15) data3[p,,c(1:10)+(i-1)*10,] <- rep(data[grid$ilon[p],grid$ilatd[p],,i,],10)
    for(i in 1:11) data3[p,,c(1:10)+(i-1+15)*10,] <- rep(data2[grid$ilon[p],grid$ilatd[p],,i,],10)
  }
  for(i in 1:260){
    # write data row by row
    no3dep[c(1:(ncell*12))+(i-1)*ncell*12] <- as.vector(t(data3[,,i,1]))
    nh4dep[c(1:(ncell*12))+(i-1)*ncell*12] <- as.vector(t(data3[,,i,2]))
  }
  
  if(T){ 
    fo <- file(paste(lpj.path,"no3_deposition_rcp",rcs[r],".clm",sep=""),"wb")
    #write header
    fwriteheader(fo,"LPJNDEP",12,1,fy=1850,ny=260)
    #write data
    writeBin(no3dep,fo,size=4)
    #close file
    close(fo)
    fo <- file(paste(lpj.path,"nh4_deposition_rcp",rcs[r],".clm",sep=""),"wb")
    #write header
    fwriteheader(fo,"LPJNDEP",12,1,fy=1850,ny=260)
    #write data
    writeBin(nh4dep,fo,size=4)
    #close file
    close(fo)
  }
  #rm(data2,data3,no3dep,nh4dep)
}

# write zero-depostion files for GGCMI CTWN-A
fo <- file(paste(lpj.path,"zero_deposition",".clm",sep=""),"wb")
#write header
fwriteheader(fo,"LPJNDEP",12,1,fy=1850,ny=260)
#write data
no3dep[] <- 0
writeBin(no3dep,fo,size=4)
#close file
close(fo)

