rm(list=ls(all=TRUE))

lpjml.pft.names <- c("trbe","trrg","tene","tebe","tesg","bne","bbsg","bnsg","c3g","c4g")


# plot N-stress as ratio of N-demand and N-uptake
require(fields)
require(maps)
#require(shape)

read.LPJmL.nc <- function(filename){
  require(ncdf4)
  nf <- nc_open(filename)
  # check for dimensions
  if(length(nf$dim)>3){
    # for PFT-specific outputs, first variable is for PFT names
    data <- ncvar_get(nf,varid=names(nf$var)[2])
  } else {
    data <- ncvar_get(nf,varid=names(nf$var))    
  }
  nc_close(nf)
  data  
}

read.LPJmL.grid <- function(filename,npix,header,swap=F){
  ff <- file(filename,"rb")
  seek(ff,where=header,origin="start")
  if(swap){
    x <- readBin(ff,integer(),size=2,n=npix*2,endian="swap")/100
  } else {
    x <- readBin(ff,integer(),size=2,n=npix*2)/100
  }
  lon <- x[(1:npix)*2-1]
  lat <- x[(1:npix)*2]
  # ilon/ilat are row and column numbers, starting at 179.75W and 89.75N as 1/1
  ilon <- as.integer((lon+180)/0.5 + 1.01)
  ilat <- as.integer((-lat+90)/0.5 + 1.01)
  area <- (111e3*0.5)*(111e3*0.5)*cos(lat/180*pi)/10000
  close(ff)
  data.frame(lon,lat,ilon,ilat,area)
}

read.LPJmL.array <- function(filename,fyear,syear,eyear,nbands,crop,npix){
  ff <- file(filename,"rb")
  yield <- array(NA,dim=c(npix,(eyear-syear+1)))
  for(y in syear:eyear){
    seek(ff,where=((y-fyear)*nbands+(crop-1))*4*npix,origin="start")
    yield[,(y-syear+1)] <- readBin(ff,double(),size=4,n=npix)
  }
  close(ff)
  yield
}

set.range <- function(x,r,nr=NA){
  if(is.na(nr)) nr<- (-r)
  x[x>r] <- r
  x[x<nr] <- nr
  x
}

# data to read
path.data <- "/p/projects/biodiversity/bloh/nitrogen/vola_cncrop/output/" 
# path to where plots should be
path.pix <- "/p/projects/macmit/users/cmueller/nitrogen/"
NPIX <- 67420
thres.stress <- 2.5
thres.flux <- 100 # check if this is a good threshold

grid <- read.LPJmL.grid(paste(path.data,"grid.bin",sep=""),NPIX,0)

for(i in 1:10){ # natural veg only
  ndemand <- rowMeans(read.LPJmL.array(paste(path.data,"pft_ndemand.bin",sep=""),1901,1990,2000,42,i,NPIX))
  nuptake <- rowMeans(read.LPJmL.array(paste(path.data,"pft_nuptake.bin",sep=""),1901,1990,2000,42,i,NPIX))
  fpc <- rowMeans(read.LPJmL.array(paste(path.data,"fpc.bin",sep=""),1901,1990,2000,11,i+1,NPIX))
  map.f <- map.d <- map.u <- array(NA,dim=c(720,360))
  for(p in 1:NPIX){
    map.d[grid$ilon[p],grid$ilat[p]] <- ndemand[p]
    map.u[grid$ilon[p],grid$ilat[p]] <- nuptake[p]
    map.f[grid$ilon[p],grid$ilat[p]] <- fpc[p]
  }
  map.s <- map.d/map.u
  map.s[map.u==0] <- NA
  map.s <- set.range(map.s,thres.stress)
  map.d <- set.range(map.d,thres.flux)
  map.u <- set.range(map.u,thres.flux)
  map.s[map.f==0] <- NA
  map.d[map.f==0] <- NA
  map.u[map.f==0] <- NA
  # plot to png
  png(paste(path.pix,"n_stress_",lpjml.pft.names[i],".png",sep=""),width=6*300,height=3*300,res=300,pointsize=6)
  par(oma=c(2,2,0,0),xpd=NA)
  split.screen( rbind(c(0,1,0.4,1), c(0,1,0,0.4)))->ind1 # big top vs. smaller bottom panel
  split.screen(c(1,2),screen=ind1[2]) -> ind3 # split bottom panel in 2
  split.screen(rbind(c(0,0.7,0,1),c(0.7,0.85,0,1),c(0.85,1,0,1)),screen=ind1[1]) -> ind2 #split large top panel in fig and 2 legend area
  screen(ind2[1]) #top left
  par(mar=c(1,1,1,1),xpd=F)
  #image.plot(dy_all_85_had_co2[,360:1],main="HadGEM, CO2",zlim=drange)
  image(x=seq(-179.75,179.75,len=720),y=seq(-89.75,89.75,len=360),axes=F,#ylim=c(-60,75),xlim=c(-160,160),
        map.s[,360:1],main=paste(lpjml.pft.names[i],"N-stress (demand/uptake)"),zlim=c(0,thres.stress),col=tim.colors(25),ylab="",xlab="")
  map(add=T,boundary=T,col="grey60",lwd=.2)
  text(-152,-30,expression(paste("A) N-STRESS")),adj=0)
  screen(ind3[1]) #bottom left
  par(mar=c(1,1,1,1),xpd=F)
  #image.plot(dy_all_85_had_co2[,360:1],main="HadGEM, CO2",zlim=drange)
  image(x=seq(-179.75,179.75,len=720),y=seq(-89.75,89.75,len=360),axes=F,#ylim=c(-60,75),xlim=c(-160,160),
        map.u[,360:1],main="",zlim=c(0,thres.flux),col=terrain.colors(25),ylab="",xlab="")
  map(add=T,boundary=T,col="grey60",lwd=.2)
  text(-160,-30,"B) N-uptake",adj=0)
  screen(ind3[2]) #bottom rigth
  par(mar=c(1,1,1,1),xpd=F)
  #image.plot(dy_all_85_had_co2[,360:1],main="HadGEM, CO2",zlim=drange)
  image(x=seq(-179.75,179.75,len=720),y=seq(-89.75,89.75,len=360),axes=F,#ylim=c(-60,75),xlim=c(-160,160),
        map.d[,360:1],main="",zlim=c(0,thres.flux),col=terrain.colors(25),ylab="",xlab="")
  map(add=T,boundary=T,col="grey60",lwd=.2)
  text(-160,-30,"C) N-demand",adj=0)
  # close screens as image.plot does not work within screens properly
  screen(ind2[2])
  par(mar=c(0,0,0,0))
  #par(xpd=NA)
  image.plot(legend.only=T,zlim=c(0,thres.stress),horizontal=F,
             smallplot=c(0.2,0.4,0.2,0.8),col=tim.colors(25),
             legend.args=list(text="N stress",side=2,line=1.5,las=3))
  #  Arrowhead(1.173,1.0,arr.type="triangle",angle=180,arr.length=0.3,arr.width=0.4,arr.col=tim.colors(25)[25],arr.lwd=1)
  screen(ind2[3])
  par(mar=c(0,0,0,0))
  image.plot(legend.only=T,zlim=c(0,thres.flux),horizontal=F,
             smallplot=c(0.2,0.4,0.2,0.8),col=terrain.colors(25),
             legend.args=list( text=expression(paste("N fluxes")), side=2, line=1.5, las=3 ))
  close.screen(all=T)
  Arrowhead(0.765,1.11,arr.type="triangle",angle=90,arr.length=0.3,arr.width=0.3,
            arr.col=tim.colors(25)[25],arr.lwd=1)
  Arrowhead(0.95,1.11,arr.type="triangle",angle=90,arr.length=0.3,arr.width=0.3,
            arr.col=terrain.colors(25)[25],arr.lwd=1)
  dev.off()
}

# plot soil cn maps ####
if(soil.c.n){
  soiln <- rowMeans(read.LPJmL.array(paste(path.data,"soiln.bin",sep=""),1901,1990,2000,1,1,NPIX))+
    rowMeans(read.LPJmL.array(paste(path.data,"litn.bin",sep=""),1901,1990,2000,1,1,NPIX))
  soilc <- rowMeans(read.LPJmL.array(paste(path.data,"soilc.bin",sep=""),1901,1990,2000,1,1,NPIX))+
    rowMeans(read.LPJmL.array(paste(path.data,"litc.bin",sep=""),1901,1990,2000,1,1,NPIX))
  map.c <- map.n <- array(NA,dim=c(720,360))
  for(p in 1:NPIX){
    map.c[grid$ilon[p],grid$ilat[p]] <- soilc[p]
    map.n[grid$ilon[p],grid$ilat[p]] <- soiln[p]
    #map.f[grid$ilon[p],grid$ilat[p]] <- fpc[p]
  }
  map.s <- map.c/map.n
  map.s[map.n==0] <- NA
  thres.cn <- 100
  thres.c <- max(abs(quantile(map.c,probs=c(0.05,0.95),na.rm=T)))
  thres.n <- max(abs(quantile(map.n,probs=c(0.05,0.95),na.rm=T)))
  
  #map.s <- set.range(map.s,thres.cn)
  map.s <- set.range(map.s,22,10)
  map.c <- set.range(map.c,thres.c)
  map.n <- set.range(map.n,thres.n)
  # plot to png
  png(paste(path.pix,"soil_cn_ratio_new_k_N.png",sep=""),width=6*300,height=3*300,res=300,pointsize=6,
      type="cairo")
  par(oma=c(2,2,0,0),xpd=NA)
  split.screen( rbind(c(0,1,0.4,1), c(0,1,0,0.4)))->ind1 # big top vs. smaller bottom panel
  split.screen(c(1,2),screen=ind1[2]) -> ind3 # split bottom panel in 2
  split.screen(rbind(c(0,0.7,0,1),c(0.7,0.8,0,1),c(0.8,0.9,0,1),c(0.9,1,0,1)),screen=ind1[1]) -> ind2 #split large top panel in fig and 2 legend area
  screen(ind2[1]) #top left
  par(mar=c(1,1,1,1),xpd=F)
  #image.plot(dy_all_85_had_co2[,360:1],main="HadGEM, CO2",zlim=drange)
  image(x=seq(-179.75,179.75,len=720),y=seq(-89.75,89.75,len=360),axes=F,#ylim=c(-60,75),xlim=c(-160,160),
        map.s[,360:1],main=paste("soil CN ratio"),zlim=c(10,22),col=tim.colors(25),ylab="",xlab="")
  map(add=T,boundary=T,col="grey60",lwd=.2)
  text(-152,-30,expression(paste("A) CN ratio in soil")),adj=0)
  screen(ind3[1]) #bottom left
  par(mar=c(1,1,1,1),xpd=F)
  #image.plot(dy_all_85_had_co2[,360:1],main="HadGEM, CO2",zlim=drange)
  image(x=seq(-179.75,179.75,len=720),y=seq(-89.75,89.75,len=360),axes=F,#ylim=c(-60,75),xlim=c(-160,160),
        map.n[,360:1],main="",zlim=c(0,thres.n),col=terrain.colors(25),ylab="",xlab="")
  map(add=T,boundary=T,col="grey60",lwd=.2)
  text(-160,-30,"B) organic soil N",adj=0)
  screen(ind3[2]) #bottom rigth
  par(mar=c(1,1,1,1),xpd=F)
  #image.plot(dy_all_85_had_co2[,360:1],main="HadGEM, CO2",zlim=drange)
  image(x=seq(-179.75,179.75,len=720),y=seq(-89.75,89.75,len=360),axes=F,#ylim=c(-60,75),xlim=c(-160,160),
        map.c[,360:1],main="",zlim=c(0,thres.c),col=terrain.colors(25),ylab="",xlab="")
  map(add=T,boundary=T,col="grey60",lwd=.2)
  text(-160,-30,"C) organic soil C",adj=0)
  # close screens as image.plot does not work within screens properly
  screen(ind2[2])
  par(mar=c(0,0,0,0))
  #par(xpd=NA)
  #image.plot(legend.only=T,zlim=c(0,thres.cn),horizontal=F,
  image.plot(legend.only=T,zlim=c(10,22),horizontal=F,
             smallplot=c(0.2,0.4,0.2,0.8),col=tim.colors(25),
             legend.args=list(text="soil CN ratio",side=2,line=1.5,las=3))
  #  Arrowhead(1.173,1.0,arr.type="triangle",angle=180,arr.length=0.3,arr.width=0.4,arr.col=tim.colors(25)[25],arr.lwd=1)
  screen(ind2[3])
  par(mar=c(0,0,0,0))
  image.plot(legend.only=T,zlim=c(0,thres.n),horizontal=F,
             smallplot=c(0.2,0.4,0.2,0.8),col=terrain.colors(25),
             legend.args=list( text=expression(paste("orgainc soil N")), side=2, line=1.5, las=3 ))
  screen(ind2[4])
  par(mar=c(0,0,0,0))
  image.plot(legend.only=T,zlim=c(0,thres.c),horizontal=F,
             smallplot=c(0.2,0.4,0.2,0.8),col=terrain.colors(25),
             legend.args=list( text=expression(paste("organic soil C")), side=2, line=1.5, las=3 ))
  close.screen(all=T)
  # Arrowhead(0.75,1.11,arr.type="triangle",angle=90,arr.length=0.3,arr.width=0.3,
  #           arr.col=tim.colors(25)[25],arr.lwd=1)
  # Arrowhead(0.865,1.11,arr.type="triangle",angle=90,arr.length=0.3,arr.width=0.3,
  #           arr.col=terrain.colors(25)[25],arr.lwd=1)
  # Arrowhead(.995,1.11,arr.type="triangle",angle=90,arr.length=0.3,arr.width=0.3,
  #           arr.col=terrain.colors(25)[25],arr.lwd=1)
  dev.off()
  
}