read_LPJmL_nc <- function(filename){
  require(ncdf4)
  nf <- nc_open(filename)
  # check for dimensions
  if(length(nf$dim)>3){
    # for PFT-specific outputs, first variable is for PFT names
    data <- ncvar_get(nf,varid=names(nf$var)[2])
  } else {
    data <- ncvar_get(nf,varid=names(nf$var))    
  }
  nc_close(nf)
  data  
}

read_LPJmL_nc_meta <- function(filename,n=NA){
  require(ncdf4)
  nf <- nc_open(filename)
  # read meta data
  #nn <- names(nf$dim)
  nn <- c("time","latitude","longitude")
  meta <- NULL
  for (i in 1:length(nn)){
    mm <- ncvar_get(nf,varid=nn[i])
    meta <- c(meta,list(mm))
  }
  names(meta) <- nn

  # check for dimensions
  if(length(nf$dim)>3){
    # for PFT-specific outputs, first variable is for PFT names
    nn <- names(nf$var)[1]
    mm <- ncvar_get(nf,varid=nn)
    if (!is.na(n)){
      ss <- nf$var[[2]]$size
      ss[3] <- 1
      data <- ncvar_get(nf,varid=names(nf$var)[2],start=c(1,1,n,1),count=ss)
      meta <- c(meta,name=list(mm[n]))
    }  else {
      data <- ncvar_get(nf,varid=names(nf$var)[2])
      meta <- c(meta,name=list(mm))
    }
  } else {
    data <- ncvar_get(nf,varid=names(nf$var))    
  }
  nc_close(nf)
  list(data=data,meta=meta)
}

read_LPJmL_bin <- function(filename,n=NA,y2=2009,y1=1901,ncell=67420,fpc=FALSE){
  nf <- file(filename,"rb")
  # check dimensions
  yl <- y2-y1+1
  sf <- file.size(filename)
  it <- sf/ncell/yl
  sb <- ifelse(it/2 > 50,4,2)
  format <- numeric()
  if (fpc){
    if (!is.na(n)) n <- n+1
    sb <- 4
  }
  np <- it/sb

  if (!is.na(n)){
    data <- array(NA,dim=c(ncell,yl))
    for (j in 1:yl){ 
      daty <- readBin(nf,format,n=ncell*np,size=sb)
      datm <- matrix(daty,ncol=np)
      data[,j] <- datm[,n]
    }
  }  else {
    data <- array(NA,dim=c(ncell,yl,np))
    for (j in 1:yl) data[,j,] <- readBin(nf,format,n=ncell*np,size=sb)
    meta <- c(meta,name=list(mm))
  }
  close(nf)
  list(data=data)
}

