rm(list=ls(all=T));gc()

#research questions####
#is the effect of tillage a function of environmental conditions?
## is the effect of tillage on yield a function of temp,nutrients and precip?--> what about other factors?
## is the effect of tillage on SOC a function of nutrients, precip, temp (and yield?)
## is the effect of tillage on NO3 a function of nutrients, precip, temp (NH3, N2O,n2?)

#stepwise regression####
#"method of fitting regression models in which the choice of predictiva variables is carrried out by an automatic procedure"
#start with all candidate variables, testing the deletion of each variable,until no further variables can be deleted without
# a statistically significant loss.

# check assumptions:
#1. are depended variables normally distributed? 
#2. assumption of linear relationship between the independent and dependent var.
#3. assumption of varience or errors is the same
#then: conduct the regression

###load libraries
library(lmtest)
library(nortest)

###general settings
#outpath <- "Z:/users/herzfeld/tillage_lpj/eval_global_rev4487_04cover/"
#outpath<-"/p/projects/macmit/users/herzfeld/tillage_lpj/eval_global_rev4487_04cover/"
outpath<- "/p/projects/macmit/users/herzfeld/tillage_lpj/eval_global_rev4487/"
ausgabe <- "/p/projects/macmit/users/felutz/evaluation_r/stepwise_fert/"
#ausgabe<- "C:/Users/felutz/Documents/2. Second article"
#lpjinput<- "D:/tillage_lpj/input_VERSION2/"
lpjinput<- "/p/projects/biodiversity/input_VERSION2/"

# temp<- paste0(lpjinput,"cru_ts3.21.1901.2012.tmp.clm")
# prec<- paste0(lpjinput,"gpcc_v6_cruts3_21_precip_1901_2010.clm")

# rad<- paste0(lpjinput, "swdown_erainterim_1901-2011.clm")
# landuse<-paste0(lpjinput,"cft1700_2005_irrigation_systems_64bands.bin")

#independens

temp<- "cru_ts3.21.1901.2012.tmp.clm" # use if readinputmonthly
temp<- paste0(lpjinput, temp) #use if read clim
prec<- "gpcc_v6_cruts3_21_precip_1901_2010.clm" # use if readinputmonthly
prec<- paste0(lpjinput, prec)#use if read clim
fert<- "fertilizer_ggcmi.clm2"
fert<- paste0(lpjinput,fert)
rad<-  "swdown_erainterim_1901-2011.clm"
rad<- paste0(lpjinput, rad)
landuse<-"cft1700_2005_irrigation_systems_64bands.bin"
landuse<- paste0(lpjinput,landuse)

#dependent

n2o_denit<- "mn2o_denit"
n2o_nit<-"mn2o_nit"
n2<- "mn2_emis"
nh3<- "mn_volatilization"
output_no3<- "soilno3"
#radl<- "lwnet_erainterim_1901-2011.clm"
#rads<- "swdown_erainterim_1901-2011.clm"
cftfrac<- "cftfrac.bin"

#settings####

scens <- c("till_nores","till_res","notill_nores","notill_res")

dirnames <- array(dim=length(scens),dimnames=list(scens))
dirnames["till_nores"] <- "out_eval_till_nores"
dirnames["till_res"] <- "out_eval_till_res"
dirnames["notill_nores"] <- "out_eval_notill_nores"
dirnames["notill_res"] <- "out_eval_notill_res"


bnames_cft <- c("wheat","rice","maize","millet","pulses","sugarbeet","cassava","sunflower","soybeans","groundnuts","rapeseed","sugarcane","others","grasses","begrasses","betrees") #16 bands
bbnames_cft<- c(paste0(bnames_cft,"_rf"),paste0(bnames_cft,"_surf"),paste0(bnames_cft,"_sprink"), paste0(bnames_cft,"_drip")) #for the 64 bands 
bnames_cft <- c(paste0(bnames_cft,"_rf"),paste0(bnames_cft,"_ir")) #32 bands
nbands_cft <- length(bnames_cft)
bnames_pft <- c("TrBEG","TrBRG","TeNEG","TeBEG","TeBSG","BorNEG","BorBSG","BorNSG","C3_GRASS","C4_GRASS",bnames_cft)
nbands_pft <- length(bnames_pft)

plotcrops <- c("wheat_rf","maize_rf","pulses_rf","rapeseed_rf")
npc <- length(plotcrops)
plotcrops_names <- array(c("rainfed wheat","rainfed maize","rainfed pulses","rainfed rapeseed"),dimnames=list(plotcrops))


NCELL <- 67420
refyear <- 1901 #start of the run
syear <- 1991 #where do you want to start to read
eyear <- 2000
nyears <- length(syear:eyear)

###functions####


read_data_cft <- function(filename="")
{
  data.out <- array(dim=c(NCELL,npc,length(scens)),dimnames=list(NULL,plotcrops,scens))
  for(scen in scens)
  {
    f <- file(paste0(outpath,dirnames[scen],"/",filename),"rb")
    seek(f,where=NCELL*(syear-refyear)*nbands_cft*4,origin="start")
    dummy <- numeric(NCELL*nbands_cft)
    for(iy in 1:nyears) dummy <- dummy + readBin(f,double(),n=NCELL*nbands_cft,size=4)
    data.out[,,scen] <- array(dummy/nyears,dim=c(NCELL,nbands_cft),dimnames=list(NULL,bnames_cft))[,plotcrops]
    close(f);rm(f,dummy);gc()
  }
  data.out
}

read_data_pft <- function(filename="")
{
  data.out <- array(dim=c(NCELL,npc,length(scens)),dimnames=list(NULL,plotcrops,scens))
  for(scen in scens)
  {
    f <- file(paste0(outpath,dirnames[scen],"/",filename),"rb")
    seek(f,where=NCELL*(syear-refyear)*nbands_pft*4,origin="start")
    dummy <- numeric(NCELL*nbands_pft)
    for(iy in 1:nyears) dummy <- dummy + readBin(f,double(),n=NCELL*nbands_pft,size=4)
    data.out[,,scen] <- array(dummy/nyears,dim=c(NCELL,nbands_pft),dimnames=list(NULL,bnames_pft))[,plotcrops]
    close(f);rm(f,dummy);gc()
  }
  data.out
}

read_data_monthly <- function(filename="")
{
  data.out <- array(dim=c(NCELL,length(scens)),dimnames=list(NULL,scens))
  for(scen in scens)
  {
    f <- file(paste0(outpath,dirnames[scen],"/",filename),"rb")
    seek(f,where=NCELL*(syear-refyear)*12*4,origin="start")
    dummy <- numeric(NCELL*12)
    for(iy in 1:nyears) dummy <- dummy + readBin(f,double(),n=NCELL*12,size=4)
    data.out[,scen] <- apply(array(dummy/nyears,dim=c(NCELL,12)),1,sum)
    close(f);rm(f,dummy);gc()
  }
  data.out
}

read_data_annual <- function(filename="")
{
  data.out <- array(data=0,dim=c(NCELL,length(scens)),dimnames=list(NULL,scens))
  for(scen in scens)
  {
    f <- file(paste0(outpath,dirnames[scen],"/",filename),"rb")
    seek(f,where=NCELL*(syear-refyear)*4,origin="start")
    for(iy in 1:nyears) data.out[,scen] <- data.out[,scen] + readBin(f,double(),n=NCELL,size=4)
    close(f)
  }
  data.out/nyears
}# 

read_input_monthly <- function(filename="",scenario=dirnames[2],opath=outpath,ncell=NCELL)
{
  data.out <- array(dim=c(NCELL,length(scenario)),dimnames=list(NULL,scenario))
  for(scen in scenario)
  {
    f <- file(paste0(outpath,scen,"/",filename),"rb")
    seek(f,where=(43+(NCELL*(syear-refyear)*12*2)),origin="start")
    dummy <- numeric(NCELL*12) #creates a vector with ncell*12 zero's
    #for all years, sum up dummy (zeroes) +  bin file of months within one year
    for(iy in 1:nyears) dummy <- dummy + readBin(f,integer(),n=NCELL*12,size=2) #adds up monthly values over all years, per grid cell
    data.out[,scen] <- apply(array(dummy/nyears,dim=c(NCELL,12)),1,sum) #1. monthly averages of all years. 2. sum up monthly average to get yearly value. 
    close(f);rm(f,dummy);gc()
  }
  data.out
}

read_input_preci <- function(filename="",ncell=NCELL)
{
  data.out <- array(dim=c(NCELL))
  f <- file(filename,"rb")
  seek(f,where=(43+(NCELL*(syear-refyear)*12*2)),origin="start")
    dummy <- numeric(NCELL*12) #creates a vector with ncell*12 zero's
    #for all years, sum up dummy (zeroes) +  bin file of months within one year
    for(iy in 1:nyears) dummy <- dummy + readBin(f,integer(),n=NCELL*12,size=2) #adds up monthly values over all years, per grid cell
    data.out <- apply(array(dummy/nyears,dim=c(NCELL,12)),1,sum) #1. monthly averages of all years. 2. sum up monthly average to get yearly value. 
    close(f);rm(f,dummy);gc()
    data.out
}


read_input_radiation <- function(filename="",ncell=NCELL)
{
  data.out <- array(dim=c(NCELL))
  f <- file(filename,"rb")
  seek(f,where=(38+(NCELL*(syear-refyear)*12*2)),origin="start")
  dummy <- numeric(NCELL*12)
  for(iy in 1:nyears) dummy <- dummy + readBin(f,integer(),n=NCELL*12,size=2)
  data.out<- apply(array(dummy/nyears,dim=c(NCELL,12)),1,sum) #needs to be checked
  close(f);rm(f,dummy);gc()
  data.out
}

read_input_temp <- function(filename="",ncell=NCELL, syear=1991, refyear=1901)
{
  data.out <- array(dim=c(NCELL))
  f <- file(filename,"rb")
  seek(f,where=(43+(NCELL*(syear-refyear)*12*2)),origin="start")
  dummy <- numeric(NCELL*12)
  for(iy in 1:nyears) dummy <- dummy + readBin(f,integer(),n=NCELL*12,size=2) #dummy: sum of montly temp over all years.
  data.out<- apply(array(dummy/(nyears*12),dim=c(NCELL,12)),1,sum)#this needs to be checked.# we want: a yearly average per cell. so we need to divide by nyears and months.
  close(f);rm(f,dummy);gc()
  data.out/10 #scalar of 10 applied
}


read_data_annual_fertil <- function(filename="",bnames_cfts=bnames_cft,ncell=NCELL)
{
  data.out <- array(data=0,dim=c(NCELL,length(bnames_cfts)),dimnames=list(NULL,bnames_cfts)) #create array with 32 crops.
  f <- file(paste0(filename),"rb")
  seek(f,where=43,origin="start") #only skip header, because we only have one year
  #for all bands, sum up dummy(zeroes) +binfile of all the crops
  data.out <- matrix(readBin(f,integer(),n=NCELL*32,size=2),ncol=length(bnames_cfts),byrow=T)
  close(f);rm(f);gc()
  data.out/100
  }#




read_data_landuse <- function(filename="",syear=2000, refyear=1700, lu=64)
{
  data.out <- array(data=0,dim=c(NCELL,lu)) #
  f <- file(filename,"rb")
  seek(f,where=43+NCELL*(syear-refyear)*2*lu,origin="start")
  #data.out <- array(readBin(f,integer(),n=NCELL*64,size=2), dim=c(NCELL,lu))
  data.out <- matrix(readBin(f,integer(),n=NCELL*64,size=2), ncol=lu,byrow=T)
  close(f)
  data.out/1000
}

wrapperlm <- function(mod){
  test <- NULL
  n <- length(mod$residuals)
  # normal distribution of residuals: Shapiro-Wilk test 
  # test value < p_value => H0 of normal distribution is rejected, i.e. the data are likely not normally distributed; this is indicated by a value of 1
  p_value <- 0.1
  if (n > 5000){
    hist(mod$residuals, breaks=50,main="N too large, visual check")
    NV <- if (ad.test(mod$residuals)$p.value < p_value) 1 else 0
  } else
  NV <- if (shapiro.test(mod$residuals)$p.value < p_value) 1 else 0
  test <- c(test,SW=NV)
  
  p_value <- 0.05
  # homoskedasticity: Breusch-Pagan test
  # test value < p_value => H0 of homoskedasticity is rejected, i.e. the data are heteroskedatisc; this is indicated by a value of 1
  #pvals <- bptest(regression_formula, data=df_gc)$p.value
  pvals <- bptest(mod, data=mod$model)$p.value
  BP <- if (pvals < p_value) 1 else 0
  test <- c(test,BP=BP)
  
  # autocorrelation: Breusch-Godfrey Test
  # test value < p_value => H0 of no serial autocorrelation is rejected, i.e. the data are serially autocorrelated; this is indicated by a value of 1
  BG <- if (bgtest(mod)$p.value < p_value) 1 else 0
  test <- c(test,BG=BG)
  
  # missing quadratic variables, i.e. wrong functional form: RESET (= Regression Equation Specification Error Test)
  # this test evaluates whether a non-linear combination of the exogenous variables has an influence on Y
  # if there is one, the model should be reformulated; this is indicated by a value of 1
  RESET <- resettest(formula=mod, power = 2:3, type = c("fitted", "regressor", "princomp"), data = df_gc)
  pRESET <- if (RESET$p.value < p_value) 1 else 0 
  test <- c(test,pR=pRESET)
  
  test  
}

lmplots<-  function(filename="",var){
  pdf(file= paste0(ausgabe,filename,".pdf",sep=""))
  layout(matrix(1:4, ncol = 2))
  plot(var)
  legend("topright",legend=wrapperlm(var), title=filename,col=red)
  layout(1)
  dev.off()
}

histograms<- function(indep=dependent) #be aware that nam might have to be changed, depending on dependent or independent
{
  #nam=c("temperature","precipitation","radiation","fertilizer") #set names for the histograms
  nam=c("yield","soc","no3")
  pdf(file= paste0(ausgabe,"hist",".pdf", sep="" ))
  
  for (i in 1:length(indep)){ #loops over the objects (which are vectors), in the dataframe
    ind=indep[i]
    #screen(ind)
    hist(indep[,i],main=nam[i],breaks=60,plot=T) #create a histogram for all the objects of the dataframe. 
    #main should return the name of the object in the list. the name of the vector
  }
  close.screen(all=T)
  dev.off()
} #end of function

#some plotting####
plotplot<- function(dep, indep= dependent)
{
  nam=c("yield","soc","no3")
  pdf(file= paste0(ausgabe,"plot",".pdf", sep="" ))
  for (i in 1:length(indep)){ #loops over the objects (which are vectors), in the dataframe
    ind=indep[i]
    #screen(ind)
    par(mfrow=c(2,2))
    plot(indep[,i]~fertilizer+precipitation+radiation+temperature, ylab=nam[i]) #create a correlation for all the objects of the dataframe. 
    
    #main should return the name of the object in the list. the name of the vector
  }
  close.screen(all=T)
  dev.off()
} 

#inputs####
#masking out 0 cropland
landuse<- read_data_landuse(landuse) #num[1:67420, 1:64]
#remove columns (bands) 13-16, 29,32, 45-48, 61-64.
#leaves 48 bands
#sum up the rows of the left over bands, leaving vector [1:67420].
landuse <- rowSums(landuse[,-c(rep(13:16,4)+rep(16*(0:3),each=4))])
#cells with (a bit of)landuse = 1 (scaling up) others are NA
landuse[landuse==0]<-NA
landuse[landuse>0]<-1

#for the inputs, we use the [,1]: we copied the 4 inputs to all scenarios to avoid
#changing the function (sorry, quick and dirty). so here we chose the 1st scenario, but the inputs are the 
#same for all scenarios of course.
temperature<- read_input_temp(temp) #average temp over all months and years.
temperature<- temperature[landuse>0]
radiation <- read_input_radiation(rad)
#radiation<- radiation[,1]
radiation<-radiation[landuse>0]
precipitation<- read_input_preci(prec)
precipitation<- precipitation[landuse>0]

#be aware; of the 32 crops.
fertilizer<- read_data_annual_fertil(fert)
#picking rf wheat!
fertilizer<- fertilizer[,1]
fertilizer<- fertilizer[landuse>0]


#outputs####
#masking out 0 cropland
#total yield for all crops, averaged over nyears: gC/m2
yield<- read_data_cft("pft_harvest.pft.bin") #[1:67420, 1:4, 1:4], chr [1:4] "wheat_rf" "maize_rf" "pulses_rf" "rapeseed_rf"["till_nores" "till_res""notill_nores" "notill_res"]
#crop specific yield, averaged over nyears: gC/m2
yield<- yield[,1,2]-yield[,1,4] #till_res- notill_res for rf wheat
yield<-yield[landuse>0]

#SOC
soc<- read_data_annual("soilc.bin")
soc<- soc[,2]-soc[,4]# soc output for till_res
soc<- soc[landuse>0]


#soil NO3
no3<- read_data_annual("soilno3.bin")
no3<- no3[,2]-no3[,4] #no3 output for till_res
no3<- no3[landuse>0]

#histograms_new####

independent<- data.frame(cbind(temperature,precipitation,radiation,fertilizer))
dependent<- data.frame(cbind(yield,soc,no3))
plotplot(yield)
histograms(dependent)

# lm function: needed as argument for lmwrapper function. dependend~dependent

#yield vs:temp, prec,rad,fert
yield_fert <- lm(yield~fertilizer)
summary(yield_fert)$r.squared
yield_prec <- lm(yield~precipitation)
summary(yield_prec)$r.squared
yield_rad <- lm(yield~radiation)
summary(yield_rad)$r.squared
yield_temp <- lm(yield~temperature)
summary(yield_temp)$r.squared


# soc vs: fert, precipitation, temp,yield, radiation
soc_fert <- lm(soc~fertilizer)
soc_prec <- lm(soc~precipitation)
soc_rad <- lm(soc~radiation)
soc_temp <- lm(soc~temperature)

# no3 vs: precipitation, temp,fert, radiation
no3_prec <- lm(no3~precipitation)
no3_rad <- lm(no3~radiation)
no3_temp <- lm(no3~temperature)
no3_fert <- lm(no3~fertilizer)
outcomes<- c("SW,BP,BG,pR")

#plotting the lm functions (q-q plot, residuals, etc into a pdf and give the results of wrapper lm in legend)
lmplots("yield_fert",yield_fert)
lmplots("yield_prec",yield_prec)
lmplots("yield_rad",yield_rad)
lmplots("yield_temp",yield_temp)

lmplots("soc_fert",soc_fert)
lmplots("soc_prec",soc_prec)
lmplots("soc_rad",soc_rad)
lmplots("soc_temp",soc_temp)

lmplots("no3_fert",no3_fert)
lmplots("no3_prec",no3_prec)
lmplots("no3_rad",no3_rad)
lmplots("no3_temp",no3_temp)

#stepwise regression####
yield_step<- lm(yield~fertilizer+precipitation+radiation+temperature)
yield_step<- step(yield_step, direction="backward")
summary(yield_step)$r.squared

soc_step<- lm(soc~fertilizer+precipitation+radiation+temperature)
soc_step<- step(soc_step, direction="backward")
summary(soc_step)$r.squared

no3_step<- lm(no3~fertilizer+precipitation+radiation+temperature)
no3_step<- step(no3_step, direction="backward")
summary(no3_step)$r.squared




