##############################################################################
##                                                                          ##
##                     t  r  y  .  R                                        ##
##                                                                          ##
##  R script calculates leaf CN ratio histogram from TRY dataset            ##
##  and fits data to lognormal distribution                                 ##
##                                                                          ##
##  written by Werner von Bloh, PIK Potsdam                                 ##
##  Last change: 09.09.2016                                                 ##
##                                                                          ##
##############################################################################

library(fitdistrplus)
require(openxlsx)

look.path <- "/p/projects/lpjml/reference_data/"
look.table <- "TRY_Categorical_Traits_Lookup_Table_2011_10_12_JK.xlsx"

lookup <- read.xlsx(paste0(look.path,look.table),sheet=1,colNames=T) [,c("AccSpeciesName","PlantGrowthForm","LeafType","LeafPhenology","PhotosyntheticPathway")]
BL.EG.tr <- unique(lookup[which((lookup[,2]=="tree"|lookup[,2]=="shrub/tree") & #allow for trees and shrub/trees
                                  lookup[,3]=="broadleaved" & # only broadleaved
                                  lookup[,4]=="evergreen" # only evergreen
                                ),1])
BL.DC.tr <- unique(lookup[which((lookup[,2]=="tree"|lookup[,2]=="shrub/tree") & #allow for trees and shrub/trees
                                  lookup[,3]=="broadleaved" & # only broadleaved
                                  lookup[,4]=="deciduous" # only deciduous
),1])
NL.EG.tr <- unique(lookup[which((lookup[,2]=="tree"|lookup[,2]=="shrub/tree") & #allow for trees and shrub/trees
                                  lookup[,3]=="needleleaved" & # only needleleaved
                                  lookup[,4]=="evergreen" # only evergreen
),1])
NL.DC.tr <- unique(lookup[which((lookup[,2]=="tree"|lookup[,2]=="shrub/tree") & #allow for trees and shrub/trees
                                  lookup[,3]=="needleleaved" & # only needleleaved
                                  lookup[,4]=="deciduous" # only deciduous
),1])
# herbaceous
C3.herb <- unique(lookup[which((lookup[,2]=="herb"|lookup[,2]=="grass"|lookup[,2]=="graminoid"|lookup[,2]=="fern") & #allow for grasses and herbs
                                  lookup[,5]=="C3"  # only C3, ignoring C3/C4 and C3/CAM and C3/C4/CAM
),1])
C4.herb <- unique(lookup[which((lookup[,2]=="herb"|lookup[,2]=="grass"|lookup[,2]=="graminoid"|lookup[,2]=="fern") & #allow for grasses and herbs
                                 (lookup[,5]=="C4"|lookup[,5]=="C3/C4"| lookup[,5]=="C3/C4/CAM") # only C4, ignoring C3/C4 and C3/CAM and C3/C4/CAM
),1])
pftname <- c("BL.EG.tr","BL.DC.tr","NL.EG.tr","NL.DC.tr","C3.herb","C4.herb")
l <- list(BL.EG.tr,BL.DC.tr,NL.EG.tr,NL.DC.tr,C3.herb,C4.herb)
#try <- read.delim(file="/p/projects/lpjml/reference_data/TRY_417/TRY_Proposal_417_DataRelease_2014_12_08.txt")
try1 <- read.delim(file="/p/projects/lpjml/reference_data/TRY_417/TRY_Proposal_417_DataRelease_2014_12_08.txt")[,-c(13,21)]
try2 <- read.delim(file="/p/projects/lpjml/reference_data/TRY_417/TRY_Proposal_417_DataRelease_2015_01_16.txt")[,-c(13,21)]
try <- rbind(try1,try2)
authors <- NULL
for(pft in 1:length(l))
{
  s <-subset(try,AccSpeciesName %in% l[[pft]]) 
  cid <- unique(s[which(s[,"TraitID"]==13 & s[,"StdValue"]!=""),5])
  nid <- unique(s[which(s[,"TraitID"]==14 & s[,"StdValue"]!=""),5])
  c_and_n_id=cid[which(cid %in% nid)]
  cat("ncid",length(cid),range(cid),"nnid",length(nid),range(nid),"c_and_n_id",length(c_and_n_id),range(c_and_n_id),"\n")
  carbon <- n <- s[1:length(c_and_n_id),] # create s-like structure with length(c_and_n_id) elements
  for(i in 1:length(c_and_n_id)){
    buf <-subset(s,TraitID==13 & ObservationID %in% c_and_n_id[i])
    carbon[i,] <-buf[1,]
    carbon[i,"StdValue"] <- mean(buf[,"StdValue"])
    buf <-subset(s,TraitID==14 & ObservationID %in% c_and_n_id[i])
    n[i,] <-buf[1,]
    n[i,"StdValue"] <- mean(buf[,"StdValue"])
  }
  #str(carbon)
  #str(n)
  cat("match?", length(carbon[,"StdValue"]),"=?=",length(n[,"StdValue"]),"\n")
  cn <-subset(s,TraitID==146 & !(ObservationID %in% c_and_n_id))
  cnratio <- c(carbon[,"StdValue"]/n[,"StdValue"],cn[,"OrigValue"])
  # remove infinite values, if carbon and n have zero elements
  cnratio <- cnratio[is.finite(cnratio)]
  #authors <- unique(c(authors,paste0(as.character(carbon[,"LastName"]),as.character(carbon[,"FileName"])),paste0(as.character(cn[,"LastName"]),as.character(carbon[,"FileName"]))))
  authors <- unique(c(authors,as.character(carbon[,"FileName"]),as.character(carbon[,"FileName"])))
  cat("PFT;",pftname[pft],"\n")
  cat("Number of leaf CN entries:",length(cnratio),"\n")
  cat("authors:",authors,"\n")
  if(length(cnratio)>0)
  {
  fit <- fitdist(cnratio,"lnorm")
  plot.legend <- "lognormal"
  denscomp(fit,xlab="leaf C:N ratios",legendtext=plot.legend)
  print(summary(fit))
  cat("CNmin=",exp(fit$estimate[1]-2*fit$estimate[2])," CNmax=",exp(fit$estimate[1]+2*fit$estimate[2]),"\n")
  cnmin<-quantile(fit,0.05)
  cnmax<-quantile(fit,0.95)
  print(cnmin)
  print(cnmax)
  }
}
