/**************************************************************************************/
/**                                                                                \n**/
/**                  c  o  n  f  .  h                                              \n**/
/**                                                                                \n**/
/**     Header for LPJ configuration file                                          \n**/
/**     Must be included by LPJmL configuration file                               \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#ifndef CONF_H
#define CONF_H

#define LPJ 0          /* LPJ simulation with natural vegetation only */
#define LPJML 1        /* LPJ simulation with managed land */
#define LPJML_IMAGE 4  /* LPJ simulation with managed land coupled to IMAGE
                          model */
#define LPJML_FMS 5  /* LPJ simulation with managed land coupled to FMS */
#define LANDUSE 1
#define NO_LANDUSE 0
#define CONST_LANDUSE 3
#define ALL_CROPS 4
#define ONLY_CROPS 5
#define NO_FIXED_SDATE 0
#define FIXED_SDATE 1
#define PRESCRIBED_SDATE 2
#define NO_IRRIGATION 0
#define LIM_IRRIGATION 1
#define POT_IRRIGATION 2
#define ALL_IRRIGATION 3
#define GRASS_HARVEST_OPTIONS 1
#define NO_GRASS_HARVEST_OPTIONS 0
#define TILLAGE 1
#define NO_TILLAGE 0
#define GRASS_FIXED_PFT 1
#define NO_GRASS_FIXED_PFT 0
#define LAIMAX_CFT 0
#define LAIMAX_INTERPOLATE 1
#define CONST_LAI_MAX 2
#define LAIMAX_PAR 3
#define NO_RESERVOIR 0
#define RESERVOIR 1
#define NO_DRAINAGE 0
#define DRAINAGE 1
#define PERM 1
#define NO_PERM 0
#define NEW_PERC 1
#define OLD_PERC 0
#define NO_WATERUSE 0
#define WATERUSE 1
#define NO_NITROGEN 0
#define NITROGEN 1
#define NO_RWMANAGEMENT 0
#define RWMANAGEMENT 1
#define RADIATION_LWDOWN 3
#define RADIATION_SWONLY 2
#define RADIATION 1
#define CLOUDINESS 0
#define ALL -1
#define RAW 0
#define CLM 1
#define CLM2 2
#define TXT 3
#define FMS 4
#define CDF 5
#define RESTART 1
#define NO_RESTART 0
#define NO_FIRE 0
#define FIRE 1
#define SPITFIRE 2
#define OLD_PHENOLOGY 0
#define NEW_PHENOLOGY 1
#define NO_POPULATION 0
#define POPULATION 1
#define NO_FIREWOOD 0
#define FIREWOOD 1
#define RANDOM_PREC 1
#define INTERPOLATE_PREC 0
#define NOUT 266            /* number of output files */
#define END -1              /* end marker for output files */
#define GRIDBASED 1         /* pft-specific outputs scaled by stand->frac */
#define PFTBASED 0          /* pft-specific outputs not scaled by stand->frac */
#define DAILY_IRRIG 1       /* daily outputs for irrigated crop */
#define DAILY_RAINFED 0     /* daily outputs for rain-fed crop */
#define CONST_PREC 2
/*monthly and annual output*/
#define GRID 0          
#define FPC 1          
#define MNPP 2          
#define MGPP 3          
#define MRH 4          
#define MTRANSP 5          
#define MRUNOFF 6          
#define MDISCHARGE 7          
#define MEVAP 8          
#define MINTERC 9          
#define MSWC1 10          
#define MSWC2 11          
#define FIREC 12          
#define FIREF 13          
#define VEGC 14          
#define SOILC 15          
#define LITC 16          
#define FLUX_ESTABC 17          
#define PFT_NPP 18          
#define PFT_GCGP 19          
#define HARVESTC 20          
#define MIRRIG 21          
#define SDATE 22          
#define PFT_HARVESTC 23          
#define PFT_RHARVESTC 24          
#define COUNTRY 25          
#define REGION 26          
#define MG_VEGC 27          
#define MG_SOILC 28
#define MG_LITC 29          
#define APREC 30          
#define INPUT_LAKE 31          
#define MRETURN_FLOW_B 32          
#define MEVAP_LAKE 33          
#define ADISCHARGE 34          
#define CFTFRAC 35          
#define CFT_CONSUMP_WATER_G 36          
#define CFT_CONSUMP_WATER_B 37          
#define PROD_TURNOVER 38          
#define DEFOREST_EMIS 39          
#define AIRRIG 40          
#define CFT_AIRRIG 41          
#define HDATE 42          
#define GROWING_PERIOD 43          
#define CFT_PET 44          
#define CFT_TRANSP 45          
#define CFT_EVAP 46          
#define CFT_INTERC 47          
#define CFT_NIR 48          
#define CFT_TEMP 49          
#define CFT_PREC 50
/* daily outputs for single crops */      
#define D_LAI 51          
#define D_PHEN 52          
#define D_CLEAF 53          
#define D_CROOT 54          
#define D_CSO 55          
#define D_CPOOL 56          
#define D_WDF 57          
#define D_GROWINGDAY 58          
#define D_PVD 59          
#define D_PHU 60          
#define D_FPHU 61          
#define D_LAIMAXAD 62          
#define D_LAINPPDEF 63          
#define D_HUSUM 64          
#define D_VDSUM 65          
#define D_WSCAL 66          
#define D_NPP 67          
#define D_GPP 68          
#define D_RD 69          
#define D_RROOT 70          
#define D_RSO 71          
#define D_RPOOL 72          
#define D_GRESP 73          
#define D_TRANS 74          
#define D_EVAP 75          
#define D_PREC 76          
#define D_PERC 77          
#define D_IRRIG 78          
#define D_W0 79          
#define D_W1 80          
#define D_WEVAP 81
#define D_WHC 82
#define D_WSAT 83
#define D_KSAT 84
#define D_HI 85
#define D_FHIOPT 86
#define D_HIMIND 87
#define D_FROOT 88
#define D_TEMP 89
#define D_SUN 90
#define D_PAR 91
#define D_DAYLENGTH 92
#define D_SWE 93
#define D_DISCHARGE 94
#define D_RUNOFF 95
#define D_RH 96
#define D_INTERC 97
#define D_NUPTAKE 98
#define D_N2O_DENIT 99
#define D_N2O_NIT 100
#define D_N2_DENIT 101
#define D_LEACHING 102
#define D_BNF 103
#define D_NLEAF 104
#define D_NROOT 105
#define D_NSO 106
#define D_NPOOL 107
#define D_NO3 108
#define D_NH4 109
#define D_NSOIL_SLOW 110
#define D_NSOIL_FAST 111
#define D_ASSIM 112
#define D_ROT_MODE 113
#define D_NLIMIT 114
#define D_VSCAL 115
#define D_PET 116      /* last daily output ID; Insert additional output before! */
#define MEVAP_RES 117
#define MPREC_RES 118
#define MFIREC 119
#define MNFIRE 120
#define MFIREDI 121
#define MFIREEMISSION 122
#define MFPAR 123
#define FLUX_FIREWOOD 124
#define RHARVEST_BURNTC 125
#define RHARVEST_BURNT_IN_FIELDC 126
#define MTEMP_IMAGE 127
#define MPREC_IMAGE 128
#define MSUN_IMAGE 129
#define MWET_IMAGE 130
#define FBURN 131
#define FTIMBER 132
#define TIMBER_HARVEST 133
#define PRODUCT_POOL_FAST 134
#define PRODUCT_POOL_SLOW 135
#define LUC_IMAGE 136
#define MAXTHAW_DEPTH 137
#define MSOILTEMP1 138
#define MSOILTEMP2 139
#define MSOILTEMP3 140
#define MSOILTEMP4 141
#define MSOILTEMP5 142
#define MSOILTEMP6 143
#define MRES_STORAGE 144
#define MRES_DEMAND 145
#define MTARGET_RELEASE 146
#define MRES_CAP 147
#define SEASONALITY 148
#define MPET 149
#define TRAD_BIOFUEL 150
#define AFRAC_WD_UNSUST 151
#define MWD_UNSUST 152
#define ACONV_LOSS_EVAP 153
#define MWD_LOCAL 154
#define MWD_NEIGHB 155
#define MWD_RES 156
#define MWD_RETURN 157
#define MCONV_LOSS_EVAP 158
#define MPREC 159
#define MRAIN 160
#define MSNOWF 161
#define MMELT 162
#define MSNOWRUNOFF 163
#define MSWE 164
#define SOILC_LAYER 165
#define MRUNOFF_SURF 166
#define MRUNOFF_LAT 167
#define MSEEPAGE 168
#define MBURNTAREA 169
#define SOILC_SLOW 170
#define MSWC3 171
#define MSWC4 172
#define MSWC5 173
#define MROOTMOIST 174
#define CFT_SRAD 175
#define CFT_ABOVEGBMC 176
#define MGCONS_RF 177
#define MGCONS_IRR 178
#define MBCONS_IRR 179
#define AWATERUSE_HIL 180
#define MWATERAMOUNT 181
#define CFT_ABOVEGBMN 182
#define PFT_HARVESTN 183
#define PFT_RHARVESTN 184
#define RHARVEST_BURNTN 185
#define RHARVEST_BURNT_IN_FIELDN 186
#define FLUX_ESTABN 187
#define VEGN 188
#define SOILN 189
#define LITN 190
#define SOILNH4 191
#define SOILNO3 192
#define HARVESTN 193
#define SOILN_LAYER 194
#define SOILN_SLOW 195
#define PFT_NUPTAKE 196
#define MNUPTAKE 197
#define MLEACHING 198
#define MN2O_DENIT 199
#define MN2O_NIT 200
#define MN2_EMIS 201
#define MBNF 202
#define PFT_NDEMAND 203
#define MN_MINERALIZATION 204
#define FIREN 205
#define MN_IMMO 206
#define PFT_CLEAF 207
#define PFT_NLEAF 208
#define PFT_VEGC 209
#define PFT_VEGN 210
#define MIRRIG_RW 211
#define MTRANSP_B 212
#define CFT_TRANSP_B 213
#define MUNMET_DEMAND 214
#define CFT_FPAR 215
#define MEVAP_B 216
#define MINTERC_B 217
#define CFT_EVAP_B 218
#define CFT_INTERC_B 219
#define CFT_RETURN_FLOW_B 220
#define CFT_IRRIG_EVENTS 221
#define CFT_CONV_LOSS_EVAP 222
#define CFT_CONV_LOSS_DRAIN 223
#define MSTOR_RETURN 224
#define MCONV_LOSS_DRAIN 225
#define ACONV_LOSS_DRAIN 226
#define NEGN_FLUXES 227
#define NEGC_FLUXES 228
#define PFT_LAIMAX 229
#define PFT_CROOT 230
#define PFT_NROOT 231
#define PFT_CSAPW 232
#define PFT_NSAPW 233
#define PFT_CHAWO 234
#define PFT_NHAWO 235
#define HDATE2 236
#define SDATE2 237
#define PFT_HARVESTC2 238
#define PFT_RHARVESTC2 239
#define GROWING_PERIOD2 240
#define CFT_PET2 241
#define CFT_TRANSP2 242
#define CFT_EVAP2 243
#define CFT_INTERC2 244
#define CFT_NIR2 245
#define CFT_TEMP2 246
#define CFT_PREC2 247
#define CFT_SRAD2 248
#define CFT_ABOVEGBMC2 249
#define CFTFRAC2 250
#define CFT_AIRRIG2 251
#define SYEAR 252
#define SYEAR2 253
#define PFT_HARVESTN2 254
#define PFT_RHARVESTN2 255
#define CFT_ABOVEGBMN2 256
#define MN_VOLATILIZATION 257
#define PFT_NLIMIT 258
#define SOILNO3_LAYER 259
#define SOILNH4_LAYER 260
#define MMG_SOILC 261
#define RUNOFF_SURF 262
#define RUNOFF_LAT 263
#define LITC_ALL 264
#define LITC_AG 265

#define RANDOM_SEED 0

#endif
