/**************************************************************************************/
/**                                                                                \n**/
/**                               g  p  .  c                                       \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Function calculates potential canopy conductance at full lead              \n**/
/**     cover                                                                      \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#include "lpj.h"

Real gp(Pft *pft,          /* PFT variables */
        Real co2,          /* atmospheric CO2 concentration (ppm) */
        Real temp,         /* temperature (deg C) */
        Real par,          /* photosynthetic active radiation flux */
        Real daylength     /* daylength (h) */
       )                   /* returns pot. canopy conduct. at full leaf cover */
{
  Real agd,adtmm,gp,rd,vmax;
  if(daylength==0)
    return 0;
  vmax=0;
  adtmm=photosynthesis(&agd,&rd,&vmax,pft->par->path,LAMBDA_OPT,
                       temp_stress(pft->par,temp,daylength),ppm2Pa(co2),
                       temp,par*pft->fpc*pft->par->alphaa,daylength);
  gp=1.6*adtmm/(ppm2bar(co2)*(1.0-LAMBDA_OPT)*hour2sec(daylength));

  return (gp<1e-20 || pft->fpc<1e-20) ? 0 : gp;
} /* of 'gp' */
