/**************************************************************************************/
/**                                                                                \n**/
/**                  i  n  t  e  r  p  o  l  a  t  e  .  c                         \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Interpolation function in order to get daily values from monthly           \n**/
/**     data                                                                       \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#include <stdio.h>
#include "types.h"
#include "date.h"

Real interpolate(const MReal mval, /* monthly values to interpolate        */
                 int month,        /* month (0..11)                        */
                 int dm            /* day of month (0..ndaymonth[month]-1) */
                )                  /* returns interpolated value           */
{
  int nextmonth;
  Real diff;
#ifdef DEBUG3
  printf("dm=%d,month=%d,ndaymonth=%d\n",dm,month,ndaymonth[month]);
#endif
  if(dm>=ndaymonth[month]/2)
  {
    dm-=ndaymonth[month]/2;
    nextmonth=(month<NMONTH-1) ? month+1 : 0;
  }
  else
  {
    nextmonth=month;
    if(month==0)
      month=NMONTH-1;
    else
      month--; 
    dm+=ndaymonth[month]/2+ndaymonth[month] %2 ; 
  }
  diff=1.0/(ndaymonth[month]/2+ndaymonth[nextmonth]/2+ndaymonth[month] %2 );
#ifdef DEBUG3
  printf("mval[%d]=%g,mval[%d]=%g,dm=%d,diffday=%g,%d\n",month,mval[month],
         nextmonth,mval[nextmonth],dm,1/diff,ndaymonth[month]);
  printf("res=%g\n",mval[month]+dm*(mval[nextmonth]-mval[month])*diff);
#endif

  return mval[month]+dm*(mval[nextmonth]-mval[month])*diff;
} /* of 'interpolate' */
