/**************************************************************************************/
/**                                                                                \n**/
/**              f  o  p  e  n  s  o  i  l  c  o  d  e  .  c                       \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Function opens soil code file                                              \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#include "lpj.h"

FILE *fopensoilcode(const Filename *filename, /* filename  of soil code file */
                    Bool *swap,           /* byte order has to be changed */
                    int *version,         /* returns version of CLM file */
                    Bool isout            /* enable output of error messages */
                   )                      /* returns file pointer or NULL */
{
  FILE *file;
  Header header;
  file=fopen(filename->name,"rb");
  if(file==NULL)
  {
    if(isout)
      printfopenerr(filename->name);
    return NULL;
  }
  if(filename->fmt==RAW)
    *swap=FALSE;
  else
  {
    if(filename->fmt==CLM)
      *version=READ_VERSION;
    else
      *version=2;
    if(freadheader(file,&header,swap,LPJSOIL_HEADER,version))
    {
      if(isout)
        fprintf(stderr,"ERROR154: Invalid header in '%s'.\n",filename->name);
      fclose(file);
      return NULL;
    }
  }
  return file;
} /* of 'fopensoilcode' */
