/**************************************************************************************/
/**                                                                                \n**/
/**                t  i  l  l  a  g  e  .  c                                       \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Function copies soil variables                                             \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#include "lpj.h"

void tillage(Stand *stand, Real frac)
{
  int i,l;

  /* move above ground litter pools to same litter structure in sub-soil (agsub) */
  for (l = 0; l < stand->soil.litter.n; l++)
  {
    stand->soil.litter.agsub[l].trait.leaf.carbon += stand->soil.litter.ag[l].trait.leaf.carbon*frac;
      stand->soil.litter.ag[l].trait.leaf.carbon *= (1-frac);
    if (stand->soil.litter.ag[l].trait.leaf.carbon < epsilon) stand->soil.litter.ag[l].trait.leaf.carbon = 0;
    stand->soil.litter.agsub[l].trait.leaf.nitrogen += stand->soil.litter.ag[l].trait.leaf.nitrogen*frac;
    stand->soil.litter.ag[l].trait.leaf.nitrogen *= (1 - frac);
    if (stand->soil.litter.ag[l].trait.leaf.nitrogen < epsilon) stand->soil.litter.ag[l].trait.leaf.nitrogen = 0;

    for (i = 0; i < NFUELCLASS; i++)
    {
        stand->soil.litter.agsub[l].trait.wood[i].carbon += stand->soil.litter.ag[l].trait.wood[i].carbon*frac;
        stand->soil.litter.ag[l].trait.wood[i].carbon *= (1 - frac);
    if (stand->soil.litter.ag[l].trait.wood[i].carbon < epsilon) stand->soil.litter.ag[l].trait.wood[i].carbon = 0;
      stand->soil.litter.agsub[l].trait.wood[i].nitrogen += stand->soil.litter.ag[l].trait.wood[i].nitrogen*frac;
      stand->soil.litter.ag[l].trait.wood[i].nitrogen *= (1 - frac);
    if (stand->soil.litter.ag[l].trait.wood[i].nitrogen < epsilon) stand->soil.litter.ag[l].trait.wood[i].nitrogen = 0;
    }
  }

  /* do some changes to bulk density here */
  for(l=0;l<NTILLLAYER;l++)
    stand->soil.df_tillage[l]-=(stand->soil.df_tillage[l]-0.667*1)*param.mixing_efficiency; /* eq. 305 from APEX documentation, expressed as fraction of BD after full settling */

  /* do whatever tillage is supposed to do otherwise here */
 
}
