/**************************************************************************************/
/**                                                                                \n**/
/**                  f  s  c  a  n  s  t  r  i  n  g  .  c                         \n**/
/**                                                                                \n**/
/**     Functions reads string of maximum length STRING_LEN. String is             \n**/
/**     read up to the first occurrence of a whitespace character or is            \n**/
/**     delimited by  '"'.                                                         \n**/
/**     Returns FALSE on success                                                   \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**************************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "types.h"

Bool fscanstring(FILE *file, /* file pointer of a text file         */
                 String s    /* pointer to a char array of dimension
                                STRING_LEN+1                        */
                )            /* return TRUE on error                */
{
  int c;
  int len;
  /* searching for first occurrence of non-whitespace character  */
  while((c=fgetc(file))!=EOF)
    if(!isspace(c))
    {
      /* non-whitespace character found */
      if(c=='\"') /* opening '"' found? */
      {
        len=0;
        while((c=fgetc(file))!=EOF)
        {
          if(c=='\"') /* closing '"' found? */
          {
            s[len]='\0';  /* yes, return with success */
            return FALSE;
          }
          else if(len==STRING_LEN)  /* string too long? */
            break;
          else if(c=='\\') /* backslash found? */
          {
            if((c=fgetc(file))==EOF) /* yes, read next character */
            {
              s[len]='\0';
              return TRUE;
            }
            else
              switch(c)
              {
                case '"': case '\\':
                  s[len++]=(char)c;
                  break;
                case 'n':
                  s[len++]='\n';
                  break;
                case 't':
                  s[len++]='\t';
                  break;
                default:
                  s[len]='\0';
                  return TRUE;
              }
          }
          else
            s[len++]=(char)c;
        }
      }
      else
      {
        s[0]=(char)c;
        len=1;
        while((c=fgetc(file))!=EOF)
        {
          if(isspace(c))
          {
            s[len]='\0';  /* yes, return with success */
            return FALSE;
          }
          else if(len==STRING_LEN)  /* string too long? */
          {
            s[len]='\0';  /* terminate string */
            return TRUE;
          }
          else
            s[len++]=(char)c;
        }
        s[len]='\0';
        return FALSE;
      }
      s[len]='\0';  /* terminate string */
      return TRUE;
    }
  *s='\0';  /* no, exits with error */
  return TRUE;
} /* of 'fscanstring' */
